from typing import Any, Dict, List, Optional, Union

from anyscale._private.models.model_base import ResultIterator
from anyscale._private.sdk import sdk_command
from anyscale.cli_logger import BlockLogger
from anyscale.job._private.job_sdk import PrivateJobSDK
from anyscale.job.models import JobConfig, JobLogMode, JobState, JobStatus


logger = BlockLogger()


def _resolve_id_from_args(
    id: Optional[str], kwargs: Dict[str, Any]  # noqa: A002
) -> Optional[str]:
    """Return the correct id as passed through id and kwargs.

    As job_id is being soft deprecated, we will warn if that is passed
    through kwargs.

    If id is passed, id will always be returned (regardless of job_id
    being passed in kwargs). If id is None and job_id is passed in kwargs,
    we will return that as the id to be used.
    """
    if "job_id" in kwargs:
        logger.warning("`job_id` has been deprecated, use `id` instead.")

    if id is not None:
        return id
    else:
        return kwargs.get("job_id")


_JOB_SDK_SINGLETON_KEY = "job_sdk"

_SUBMIT_EXAMPLE = """
import anyscale
from anyscale.job.models import JobConfig

anyscale.job.submit(
    JobConfig(
        name="my-job",
        entrypoint="python main.py",
        working_dir=".",
    ),
)
"""

_SUBMIT_ARG_DOCSTRINGS = {"config": "The config options defining the job."}


@sdk_command(
    _JOB_SDK_SINGLETON_KEY,
    PrivateJobSDK,
    doc_py_example=_SUBMIT_EXAMPLE,
    arg_docstrings=_SUBMIT_ARG_DOCSTRINGS,
)
def submit(config: JobConfig, *, _private_sdk: Optional[PrivateJobSDK] = None) -> str:
    """Submit a job.

    Returns the id of the submitted job.
    """
    return _private_sdk.submit(config)  # type: ignore


_STATUS_EXAMPLE = """
import anyscale
from anyscale.job.models import JobStatus

status: JobStatus = anyscale.job.status(name="my-job")
"""

_STATUS_ARG_DOCSTRINGS = {
    "name": "Name of the job.",
    "id": "Unique ID of the job",
    "cloud": "The Anyscale Cloud to run this workload on. If not provided, the organization default will be used (or, if running in a workspace, the cloud of the workspace).",
    "project": "Named project to use for the job. If not provided, the default project for the cloud will be used (or, if running in a workspace, the project of the workspace).",
}


@sdk_command(
    _JOB_SDK_SINGLETON_KEY,
    PrivateJobSDK,
    doc_py_example=_STATUS_EXAMPLE,
    arg_docstrings=_STATUS_ARG_DOCSTRINGS,
)
def status(
    *,
    name: Optional[str] = None,
    id: Optional[str] = None,  # noqa: A002
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    _private_sdk: Optional[PrivateJobSDK] = None,
    **_kwargs: Dict[str, Any],
) -> JobStatus:
    """Get the status of a job."""
    id = _resolve_id_from_args(id, _kwargs)  # noqa: A001
    return _private_sdk.status(name=name, job_id=id, cloud=cloud, project=project)  # type: ignore


_TERMINATE_EXAMPLE = """
import anyscale

anyscale.job.terminate(name="my-job")
"""

_TERMINATE_ARG_DOCSTRINGS = {
    "name": "Name of the job.",
    "id": "Unique ID of the job",
    "cloud": "The Anyscale Cloud to run this workload on. If not provided, the organization default will be used (or, if running in a workspace, the cloud of the workspace).",
    "project": "Named project to use for the job. If not provided, the default project for the cloud will be used (or, if running in a workspace, the project of the workspace).",
}


@sdk_command(
    _JOB_SDK_SINGLETON_KEY,
    PrivateJobSDK,
    doc_py_example=_TERMINATE_EXAMPLE,
    arg_docstrings=_TERMINATE_ARG_DOCSTRINGS,
)
def terminate(
    *,
    name: Optional[str] = None,
    id: Optional[str] = None,  # noqa: A002
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    _private_sdk: Optional[PrivateJobSDK] = None,
    **_kwargs: Dict[str, Any],
) -> str:
    """Terminate a job.

    This command is asynchronous, so it always returns immediately.

    Returns the id of the terminated job.
    """
    id = _resolve_id_from_args(id, _kwargs)  # noqa: A001
    return _private_sdk.terminate(name=name, job_id=id, cloud=cloud, project=project)  # type: ignore


_ARCHIVE_EXAMPLE = """
import anyscale

anyscale.job.archive(name="my-job")
"""

_ARCHIVE_ARG_DOCSTRINGS = {
    "name": "Name of the job.",
    "id": "Unique ID of the job",
    "cloud": "The Anyscale Cloud to run this workload on. If not provided, the organization default will be used (or, if running in a workspace, the cloud of the workspace).",
    "project": "Named project to use for the job . If not provided, the default project for the cloud will be used (or, if running in a workspace, the project of the workspace).",
}


@sdk_command(
    _JOB_SDK_SINGLETON_KEY,
    PrivateJobSDK,
    doc_py_example=_ARCHIVE_EXAMPLE,
    arg_docstrings=_ARCHIVE_ARG_DOCSTRINGS,
)
def archive(
    *,
    name: Optional[str] = None,
    id: Optional[str] = None,  # noqa: A002
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    _private_sdk: Optional[PrivateJobSDK] = None,
    **_kwargs: Dict[str, Any],
) -> str:
    """Archive a job.

    This command is asynchronous, so it always returns immediately.

    Returns the id of the archived job.
    """
    id = _resolve_id_from_args(id, _kwargs)  # noqa: A001
    return _private_sdk.archive(name=name, job_id=id, cloud=cloud, project=project)  # type: ignore


_WAIT_EXAMPLE = """\
import anyscale

anyscale.job.wait(name="my-job", timeout_s=180)"""

_WAIT_ARG_DOCSTRINGS = {
    "name": "Name of the job.",
    "id": "Unique ID of the job",
    "cloud": "The Anyscale Cloud to run this workload on. If not provided, the organization default will be used (or, if running in a workspace, the cloud of the workspace).",
    "project": "Named project to use for the job. If not provided, the default project for the cloud will be used (or, if running in a workspace, the project of the workspace).",
    "state": "Target state of the job",
    "timeout_s": "Number of seconds to wait before timing out, this timeout will not affect job execution",
    "follow": "Whether to follow the logs of the job. If True, the logs will be streamed to the console.",
}


@sdk_command(
    _JOB_SDK_SINGLETON_KEY,
    PrivateJobSDK,
    doc_py_example=_WAIT_EXAMPLE,
    arg_docstrings=_WAIT_ARG_DOCSTRINGS,
)
def wait(
    *,
    name: Optional[str] = None,
    id: Optional[str] = None,  # noqa: A002
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    state: Union[JobState, str] = JobState.SUCCEEDED,
    timeout_s: float = 1800,
    follow: bool = False,
    _private_sdk: Optional[PrivateJobSDK] = None,
    **_kwargs: Dict[str, Any],
):
    """"Wait for a job to enter a specific state."""
    id = _resolve_id_from_args(id, _kwargs)  # noqa: A001
    _private_sdk.wait(  # type: ignore
        name=name,
        job_id=id,
        cloud=cloud,
        project=project,
        state=state,
        timeout_s=timeout_s,
        follow=follow,
    )


_GET_LOGS_EXAMPLE = """\
import anyscale

anyscale.job.get_logs(name="my-job", run="job-run-name")
"""

_GET_LOGS_ARG_DOCSTRINGS = {
    "name": "Name of the job",
    "id": "Unique ID of the job",
    "cloud": "The Anyscale Cloud to run this workload on. If not provided, the organization default will be used (or, if running in a workspace, the cloud of the workspace).",
    "project": "Named project to use for the job. If not provided, the default project for the cloud will be used (or, if running in a workspace, the project of the workspace).",
    "run": "The name of the run to query. Names can be found in the JobStatus. If not provided, the last job run will be used.",
    "mode": "The mode of log fetching to be used. Supported modes can be found in JobLogMode. If not provided, JobLogMode.TAIL will be used.",
    "max_lines": "The number of log lines to be fetched. If not provided, the complete log will be fetched.",
}


@sdk_command(
    _JOB_SDK_SINGLETON_KEY,
    PrivateJobSDK,
    doc_py_example=_GET_LOGS_EXAMPLE,
    arg_docstrings=_GET_LOGS_ARG_DOCSTRINGS,
)
def get_logs(
    *,
    id: Optional[str] = None,  # noqa: A002
    name: Optional[str] = None,
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    run: Optional[str] = None,
    mode: Union[str, JobLogMode] = JobLogMode.TAIL,
    max_lines: Optional[int] = None,
    _private_sdk: Optional[PrivateJobSDK] = None,
    **_kwargs: Dict[str, Any],
) -> str:
    """Query the jobs for a job run."""
    id = _resolve_id_from_args(id, _kwargs)  # noqa: A001
    return _private_sdk.get_logs(  # type: ignore
        job_id=id,
        name=name,
        cloud=cloud,
        project=project,
        run=run,
        mode=mode,
        max_lines=max_lines,
    )


_ADD_TAGS_EXAMPLE = """
import anyscale

anyscale.job.add_tags(id="job_123", tags={"team": "mlops", "env": "prod"})
"""

_ADD_TAGS_ARG_DOCSTRINGS = {
    "job_id": "ID of the job. Provide either job_id or name.",
    "name": "Name of the job. Provide either job_id or name.",
    "cloud": "Cloud name (used when resolving by name).",
    "project": "Project name (used when resolving by name).",
    "tags": "Key/value tags to upsert as a map {key: value}.",
}

_REMOVE_TAGS_EXAMPLE = """
import anyscale

anyscale.job.remove_tags(id="job_123", keys=["team", "env"])
"""

_REMOVE_TAGS_ARG_DOCSTRINGS = {
    "job_id": "ID of the job. Provide either job_id or name.",
    "name": "Name of the job. Provide either job_id or name.",
    "cloud": "Cloud name (used when resolving by name).",
    "project": "Project name (used when resolving by name).",
    "keys": "List of tag keys to remove.",
}


@sdk_command(
    _JOB_SDK_SINGLETON_KEY,
    PrivateJobSDK,
    doc_py_example=_ADD_TAGS_EXAMPLE,
    arg_docstrings=_ADD_TAGS_ARG_DOCSTRINGS,
)
def add_tags(
    *,
    job_id: Optional[str] = None,
    name: Optional[str] = None,
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    tags: Dict[str, str],
    _private_sdk: Optional[PrivateJobSDK] = None,
):
    """Upsert (add/update) tag key/value pairs for a job."""
    return _private_sdk.add_tags(  # type: ignore
        job_id=job_id, name=name, cloud=cloud, project=project, tags=tags
    )


@sdk_command(
    _JOB_SDK_SINGLETON_KEY,
    PrivateJobSDK,
    doc_py_example=_REMOVE_TAGS_EXAMPLE,
    arg_docstrings=_REMOVE_TAGS_ARG_DOCSTRINGS,
)
def remove_tags(
    *,
    job_id: Optional[str] = None,
    name: Optional[str] = None,
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    keys: List[str],
    _private_sdk: Optional[PrivateJobSDK] = None,
):
    """Remove tags by key from a job."""
    return _private_sdk.remove_tags(  # type: ignore
        job_id=job_id, name=name, cloud=cloud, project=project, keys=keys
    )


_LIST_TAGS_EXAMPLE = """
import anyscale

tags: dict[str, str] = anyscale.job.list_tags(name="my-job")
"""

_LIST_TAGS_ARG_DOCSTRINGS = {
    "job_id": "ID of the job. Provide either job_id or name.",
    "name": "Name of the job. Provide either job_id or name.",
    "cloud": "Cloud name (used when resolving by name).",
    "project": "Project name (used when resolving by name).",
}


@sdk_command(
    _JOB_SDK_SINGLETON_KEY,
    PrivateJobSDK,
    doc_py_example=_LIST_TAGS_EXAMPLE,
    arg_docstrings=_LIST_TAGS_ARG_DOCSTRINGS,
)
def list_tags(
    *,
    job_id: Optional[str] = None,
    name: Optional[str] = None,
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    _private_sdk: Optional[PrivateJobSDK] = None,
) -> Dict[str, str]:
    """List tags for a job as a key/value mapping."""
    return _private_sdk.list_tags(  # type: ignore
        job_id=job_id, name=name, cloud=cloud, project=project
    )


_LIST_EXAMPLE = """
import anyscale
from anyscale.job.models import JobStatus

# List all jobs
for job in anyscale.job.list(max_items=10):
    print(f"{job.name}: {job.state}")

# Filter by project
jobs = list(anyscale.job.list(project="my-project"))
"""

_LIST_ARG_DOCSTRINGS = {
    "name": "Filter by job name.",
    "job_id": "Fetch a specific job by ID.",
    "project": "Filter by project name.",
    "cloud": "Filter by cloud name.",
    "include_all_users": "Include jobs from all users.",
    "include_archived": "Include archived jobs.",
    "state_filter": "Filter by job states (list of JobState or str).",
    "tags_filter": "Filter by tags (dict of key to list of values).",
    "page_size": "Number of items per page.",
    "max_items": "Maximum total items to return.",
}


@sdk_command(
    _JOB_SDK_SINGLETON_KEY,
    PrivateJobSDK,
    doc_py_example=_LIST_EXAMPLE,
    arg_docstrings=_LIST_ARG_DOCSTRINGS,
)
def list(  # noqa: A001, PLR0913
    *,
    name: Optional[str] = None,
    job_id: Optional[str] = None,
    project: Optional[str] = None,
    cloud: Optional[str] = None,
    include_all_users: bool = False,
    include_archived: bool = False,
    state_filter: Optional[List[Union[JobState, str]]] = None,
    tags_filter: Optional[Dict[str, List[str]]] = None,
    page_size: Optional[int] = None,
    max_items: Optional[int] = None,
    _private_sdk: Optional[PrivateJobSDK] = None,
) -> ResultIterator[JobStatus]:
    """List jobs with filtering and pagination.

    Returns a ResultIterator that lazily fetches pages of jobs.
    """
    return _private_sdk.list(  # type: ignore
        name=name,
        job_id=job_id,
        project=project,
        cloud=cloud,
        include_all_users=include_all_users,
        include_archived=include_archived,
        state_filter=state_filter,
        tags_filter=tags_filter,
        page_size=page_size,
        max_items=max_items,
    )



# coding: utf-8

"""
    Anyscale API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from anyscale_client.configuration import Configuration


class PhysicalResources(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'cpu': 'int',
        'gpu': 'int',
        'memory': 'int',
        'tpu': 'int',
        'accelerator': 'str',
        'anyscale_tpu_hosts': 'int',
        'cpu_architecture': 'str'
    }

    attribute_map = {
        'cpu': 'cpu',
        'gpu': 'gpu',
        'memory': 'memory',
        'tpu': 'tpu',
        'accelerator': 'accelerator',
        'anyscale_tpu_hosts': 'anyscale/tpu_hosts',
        'cpu_architecture': 'cpu_architecture'
    }

    def __init__(self, cpu=None, gpu=None, memory=None, tpu=None, accelerator=None, anyscale_tpu_hosts=None, cpu_architecture=None, local_vars_configuration=None):  # noqa: E501
        """PhysicalResources - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._cpu = None
        self._gpu = None
        self._memory = None
        self._tpu = None
        self._accelerator = None
        self._anyscale_tpu_hosts = None
        self._cpu_architecture = None
        self.discriminator = None

        if cpu is not None:
            self.cpu = cpu
        if gpu is not None:
            self.gpu = gpu
        if memory is not None:
            self.memory = memory
        if tpu is not None:
            self.tpu = tpu
        if accelerator is not None:
            self.accelerator = accelerator
        if anyscale_tpu_hosts is not None:
            self.anyscale_tpu_hosts = anyscale_tpu_hosts
        if cpu_architecture is not None:
            self.cpu_architecture = cpu_architecture

    @property
    def cpu(self):
        """Gets the cpu of this PhysicalResources.  # noqa: E501

        Number of CPUs to allocate.  # noqa: E501

        :return: The cpu of this PhysicalResources.  # noqa: E501
        :rtype: int
        """
        return self._cpu

    @cpu.setter
    def cpu(self, cpu):
        """Sets the cpu of this PhysicalResources.

        Number of CPUs to allocate.  # noqa: E501

        :param cpu: The cpu of this PhysicalResources.  # noqa: E501
        :type: int
        """

        self._cpu = cpu

    @property
    def gpu(self):
        """Gets the gpu of this PhysicalResources.  # noqa: E501

        Number of GPUs to allocate.  # noqa: E501

        :return: The gpu of this PhysicalResources.  # noqa: E501
        :rtype: int
        """
        return self._gpu

    @gpu.setter
    def gpu(self, gpu):
        """Sets the gpu of this PhysicalResources.

        Number of GPUs to allocate.  # noqa: E501

        :param gpu: The gpu of this PhysicalResources.  # noqa: E501
        :type: int
        """

        self._gpu = gpu

    @property
    def memory(self):
        """Gets the memory of this PhysicalResources.  # noqa: E501

        Amount of memory in bytes to allocate.  # noqa: E501

        :return: The memory of this PhysicalResources.  # noqa: E501
        :rtype: int
        """
        return self._memory

    @memory.setter
    def memory(self, memory):
        """Sets the memory of this PhysicalResources.

        Amount of memory in bytes to allocate.  # noqa: E501

        :param memory: The memory of this PhysicalResources.  # noqa: E501
        :type: int
        """

        self._memory = memory

    @property
    def tpu(self):
        """Gets the tpu of this PhysicalResources.  # noqa: E501

        Number of TPUs to allocate.  # noqa: E501

        :return: The tpu of this PhysicalResources.  # noqa: E501
        :rtype: int
        """
        return self._tpu

    @tpu.setter
    def tpu(self, tpu):
        """Sets the tpu of this PhysicalResources.

        Number of TPUs to allocate.  # noqa: E501

        :param tpu: The tpu of this PhysicalResources.  # noqa: E501
        :type: int
        """

        self._tpu = tpu

    @property
    def accelerator(self):
        """Gets the accelerator of this PhysicalResources.  # noqa: E501

        Type of accelerator (e.g., 'T4', 'L4', 'H100').  # noqa: E501

        :return: The accelerator of this PhysicalResources.  # noqa: E501
        :rtype: str
        """
        return self._accelerator

    @accelerator.setter
    def accelerator(self, accelerator):
        """Sets the accelerator of this PhysicalResources.

        Type of accelerator (e.g., 'T4', 'L4', 'H100').  # noqa: E501

        :param accelerator: The accelerator of this PhysicalResources.  # noqa: E501
        :type: str
        """

        self._accelerator = accelerator

    @property
    def anyscale_tpu_hosts(self):
        """Gets the anyscale_tpu_hosts of this PhysicalResources.  # noqa: E501

        Number of TPU hosts (for anyscale/tpu_hosts custom resource).  # noqa: E501

        :return: The anyscale_tpu_hosts of this PhysicalResources.  # noqa: E501
        :rtype: int
        """
        return self._anyscale_tpu_hosts

    @anyscale_tpu_hosts.setter
    def anyscale_tpu_hosts(self, anyscale_tpu_hosts):
        """Sets the anyscale_tpu_hosts of this PhysicalResources.

        Number of TPU hosts (for anyscale/tpu_hosts custom resource).  # noqa: E501

        :param anyscale_tpu_hosts: The anyscale_tpu_hosts of this PhysicalResources.  # noqa: E501
        :type: int
        """

        self._anyscale_tpu_hosts = anyscale_tpu_hosts

    @property
    def cpu_architecture(self):
        """Gets the cpu_architecture of this PhysicalResources.  # noqa: E501

        CPU architecture type. Valid values: 'x86_64' (default), 'arm64'.  # noqa: E501

        :return: The cpu_architecture of this PhysicalResources.  # noqa: E501
        :rtype: str
        """
        return self._cpu_architecture

    @cpu_architecture.setter
    def cpu_architecture(self, cpu_architecture):
        """Sets the cpu_architecture of this PhysicalResources.

        CPU architecture type. Valid values: 'x86_64' (default), 'arm64'.  # noqa: E501

        :param cpu_architecture: The cpu_architecture of this PhysicalResources.  # noqa: E501
        :type: str
        """

        self._cpu_architecture = cpu_architecture

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, PhysicalResources):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, PhysicalResources):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class LogFilter(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'node_ip': 'str',
        'actor_id': 'str',
        'task_id': 'str',
        'glob': 'str',
        'cluster_id': 'str',
        'job_id': 'str',
        'job_run': 'str',
        'instance_id': 'str',
        'node_type': 'NodeType',
        'process_id': 'str',
        'session_id': 'str',
        'file_name': 'str'
    }

    attribute_map = {
        'node_ip': 'node_ip',
        'actor_id': 'actor_id',
        'task_id': 'task_id',
        'glob': 'glob',
        'cluster_id': 'cluster_id',
        'job_id': 'job_id',
        'job_run': 'job_run',
        'instance_id': 'instance_id',
        'node_type': 'node_type',
        'process_id': 'process_id',
        'session_id': 'session_id',
        'file_name': 'file_name'
    }

    def __init__(self, node_ip=None, actor_id=None, task_id=None, glob=None, cluster_id=None, job_id=None, job_run=None, instance_id=None, node_type=None, process_id=None, session_id=None, file_name=None, local_vars_configuration=None):  # noqa: E501
        """LogFilter - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._node_ip = None
        self._actor_id = None
        self._task_id = None
        self._glob = None
        self._cluster_id = None
        self._job_id = None
        self._job_run = None
        self._instance_id = None
        self._node_type = None
        self._process_id = None
        self._session_id = None
        self._file_name = None
        self.discriminator = None

        if node_ip is not None:
            self.node_ip = node_ip
        if actor_id is not None:
            self.actor_id = actor_id
        if task_id is not None:
            self.task_id = task_id
        if glob is not None:
            self.glob = glob
        if cluster_id is not None:
            self.cluster_id = cluster_id
        if job_id is not None:
            self.job_id = job_id
        if job_run is not None:
            self.job_run = job_run
        if instance_id is not None:
            self.instance_id = instance_id
        if node_type is not None:
            self.node_type = node_type
        if process_id is not None:
            self.process_id = process_id
        if session_id is not None:
            self.session_id = session_id
        if file_name is not None:
            self.file_name = file_name

    @property
    def node_ip(self):
        """Gets the node_ip of this LogFilter.  # noqa: E501

        Filter logs by node_ip (Internal Node IP).  # noqa: E501

        :return: The node_ip of this LogFilter.  # noqa: E501
        :rtype: str
        """
        return self._node_ip

    @node_ip.setter
    def node_ip(self, node_ip):
        """Sets the node_ip of this LogFilter.

        Filter logs by node_ip (Internal Node IP).  # noqa: E501

        :param node_ip: The node_ip of this LogFilter.  # noqa: E501
        :type: str
        """

        self._node_ip = node_ip

    @property
    def actor_id(self):
        """Gets the actor_id of this LogFilter.  # noqa: E501

        Filter logs by actor_id.  # noqa: E501

        :return: The actor_id of this LogFilter.  # noqa: E501
        :rtype: str
        """
        return self._actor_id

    @actor_id.setter
    def actor_id(self, actor_id):
        """Sets the actor_id of this LogFilter.

        Filter logs by actor_id.  # noqa: E501

        :param actor_id: The actor_id of this LogFilter.  # noqa: E501
        :type: str
        """

        self._actor_id = actor_id

    @property
    def task_id(self):
        """Gets the task_id of this LogFilter.  # noqa: E501

        Filter logs by task_id.  # noqa: E501

        :return: The task_id of this LogFilter.  # noqa: E501
        :rtype: str
        """
        return self._task_id

    @task_id.setter
    def task_id(self, task_id):
        """Sets the task_id of this LogFilter.

        Filter logs by task_id.  # noqa: E501

        :param task_id: The task_id of this LogFilter.  # noqa: E501
        :type: str
        """

        self._task_id = task_id

    @property
    def glob(self):
        """Gets the glob of this LogFilter.  # noqa: E501

        Filter logs by a glob filter.  # noqa: E501

        :return: The glob of this LogFilter.  # noqa: E501
        :rtype: str
        """
        return self._glob

    @glob.setter
    def glob(self, glob):
        """Sets the glob of this LogFilter.

        Filter logs by a glob filter.  # noqa: E501

        :param glob: The glob of this LogFilter.  # noqa: E501
        :type: str
        """

        self._glob = glob

    @property
    def cluster_id(self):
        """Gets the cluster_id of this LogFilter.  # noqa: E501

        Filter logs by cluster_id.  # noqa: E501

        :return: The cluster_id of this LogFilter.  # noqa: E501
        :rtype: str
        """
        return self._cluster_id

    @cluster_id.setter
    def cluster_id(self, cluster_id):
        """Sets the cluster_id of this LogFilter.

        Filter logs by cluster_id.  # noqa: E501

        :param cluster_id: The cluster_id of this LogFilter.  # noqa: E501
        :type: str
        """

        self._cluster_id = cluster_id

    @property
    def job_id(self):
        """Gets the job_id of this LogFilter.  # noqa: E501

        Filter logs by job_id (Anyscale Job ID).  # noqa: E501

        :return: The job_id of this LogFilter.  # noqa: E501
        :rtype: str
        """
        return self._job_id

    @job_id.setter
    def job_id(self, job_id):
        """Sets the job_id of this LogFilter.

        Filter logs by job_id (Anyscale Job ID).  # noqa: E501

        :param job_id: The job_id of this LogFilter.  # noqa: E501
        :type: str
        """

        self._job_id = job_id

    @property
    def job_run(self):
        """Gets the job_run of this LogFilter.  # noqa: E501

        Filter logs by job_run_id (Anyscale Job Run ID).  # noqa: E501

        :return: The job_run of this LogFilter.  # noqa: E501
        :rtype: str
        """
        return self._job_run

    @job_run.setter
    def job_run(self, job_run):
        """Sets the job_run of this LogFilter.

        Filter logs by job_run_id (Anyscale Job Run ID).  # noqa: E501

        :param job_run: The job_run of this LogFilter.  # noqa: E501
        :type: str
        """

        self._job_run = job_run

    @property
    def instance_id(self):
        """Gets the instance_id of this LogFilter.  # noqa: E501

        Filter logs by instance_id (Cloud Provider Instance ID).  # noqa: E501

        :return: The instance_id of this LogFilter.  # noqa: E501
        :rtype: str
        """
        return self._instance_id

    @instance_id.setter
    def instance_id(self, instance_id):
        """Sets the instance_id of this LogFilter.

        Filter logs by instance_id (Cloud Provider Instance ID).  # noqa: E501

        :param instance_id: The instance_id of this LogFilter.  # noqa: E501
        :type: str
        """

        self._instance_id = instance_id

    @property
    def node_type(self):
        """Gets the node_type of this LogFilter.  # noqa: E501

        Filter logs by node_type (head-node, worker-nodes).  # noqa: E501

        :return: The node_type of this LogFilter.  # noqa: E501
        :rtype: NodeType
        """
        return self._node_type

    @node_type.setter
    def node_type(self, node_type):
        """Sets the node_type of this LogFilter.

        Filter logs by node_type (head-node, worker-nodes).  # noqa: E501

        :param node_type: The node_type of this LogFilter.  # noqa: E501
        :type: NodeType
        """

        self._node_type = node_type

    @property
    def process_id(self):
        """Gets the process_id of this LogFilter.  # noqa: E501

        Filter logs by process_id.  # noqa: E501

        :return: The process_id of this LogFilter.  # noqa: E501
        :rtype: str
        """
        return self._process_id

    @process_id.setter
    def process_id(self, process_id):
        """Sets the process_id of this LogFilter.

        Filter logs by process_id.  # noqa: E501

        :param process_id: The process_id of this LogFilter.  # noqa: E501
        :type: str
        """

        self._process_id = process_id

    @property
    def session_id(self):
        """Gets the session_id of this LogFilter.  # noqa: E501

        Filter logs by session_id.  # noqa: E501

        :return: The session_id of this LogFilter.  # noqa: E501
        :rtype: str
        """
        return self._session_id

    @session_id.setter
    def session_id(self, session_id):
        """Sets the session_id of this LogFilter.

        Filter logs by session_id.  # noqa: E501

        :param session_id: The session_id of this LogFilter.  # noqa: E501
        :type: str
        """

        self._session_id = session_id

    @property
    def file_name(self):
        """Gets the file_name of this LogFilter.  # noqa: E501

        Filter logs by log file name.  # noqa: E501

        :return: The file_name of this LogFilter.  # noqa: E501
        :rtype: str
        """
        return self._file_name

    @file_name.setter
    def file_name(self, file_name):
        """Sets the file_name of this LogFilter.

        Filter logs by log file name.  # noqa: E501

        :param file_name: The file_name of this LogFilter.  # noqa: E501
        :type: str
        """

        self._file_name = file_name

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, LogFilter):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, LogFilter):
            return True

        return self.to_dict() != other.to_dict()



from dataclasses import dataclass, field
from datetime import datetime

from anyscale._private.models import ModelBase


@dataclass(frozen=True)
class OrganizationInvitation(ModelBase):
    """Organization invitation model.
    """

    __doc_py_example__ = """\
import anyscale
from anyscale.organization_invitation.models import OrganizationInvitation

organization_invitations: List[OrganizationInvitation] = anyscale.organization_invitation.list()
"""

    id: str = field(metadata={"docstring": "ID of the organization invitation."},)

    def _validate_id(self, id: str):  # noqa: A002
        if not isinstance(id, str):
            raise TypeError("id must be a string.")

    email: str = field(metadata={"docstring": "Email of the organization invitation."})

    def _validate_email(self, email: str):
        if not isinstance(email, str):
            raise TypeError("email must be a string.")

    created_at: datetime = field(
        metadata={"docstring": "Creation time of the organization invitation."},
    )

    def _validate_created_at(self, created_at: datetime):
        if not isinstance(created_at, datetime):
            raise TypeError("created_at must be a datetime.")

    expires_at: datetime = field(
        metadata={"docstring": "Expiration time of the organization invitation."},
    )

    def _validate_expires_at(self, expires_at: datetime):
        if not isinstance(expires_at, datetime):
            raise TypeError("expires_at must be a datetime.")



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class CreateWorkspaceTemplate(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'name': 'str',
        'description': 'str',
        'mins_to_complete': 'int',
        'icon_type': 'WorkspaceTemplateIconType',
        'icon_bg_color': 'WorkspaceTemplateIconBackgroundColor',
        'complexity': 'WorkspaceTemplateComplexity',
        'category': 'WorkspaceTemplateCategory',
        'labels': 'list[str]',
        'published_version_id': 'str',
        'cloud_id': 'str',
        'is_global': 'bool'
    }

    attribute_map = {
        'name': 'name',
        'description': 'description',
        'mins_to_complete': 'mins_to_complete',
        'icon_type': 'icon_type',
        'icon_bg_color': 'icon_bg_color',
        'complexity': 'complexity',
        'category': 'category',
        'labels': 'labels',
        'published_version_id': 'published_version_id',
        'cloud_id': 'cloud_id',
        'is_global': 'is_global'
    }

    def __init__(self, name=None, description=None, mins_to_complete=None, icon_type=None, icon_bg_color=None, complexity=None, category=None, labels=None, published_version_id=None, cloud_id=None, is_global=False, local_vars_configuration=None):  # noqa: E501
        """CreateWorkspaceTemplate - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._name = None
        self._description = None
        self._mins_to_complete = None
        self._icon_type = None
        self._icon_bg_color = None
        self._complexity = None
        self._category = None
        self._labels = None
        self._published_version_id = None
        self._cloud_id = None
        self._is_global = None
        self.discriminator = None

        self.name = name
        if description is not None:
            self.description = description
        if mins_to_complete is not None:
            self.mins_to_complete = mins_to_complete
        if icon_type is not None:
            self.icon_type = icon_type
        if icon_bg_color is not None:
            self.icon_bg_color = icon_bg_color
        if complexity is not None:
            self.complexity = complexity
        if category is not None:
            self.category = category
        if labels is not None:
            self.labels = labels
        if published_version_id is not None:
            self.published_version_id = published_version_id
        self.cloud_id = cloud_id
        if is_global is not None:
            self.is_global = is_global

    @property
    def name(self):
        """Gets the name of this CreateWorkspaceTemplate.  # noqa: E501

        Name of the template  # noqa: E501

        :return: The name of this CreateWorkspaceTemplate.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this CreateWorkspaceTemplate.

        Name of the template  # noqa: E501

        :param name: The name of this CreateWorkspaceTemplate.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def description(self):
        """Gets the description of this CreateWorkspaceTemplate.  # noqa: E501

        Description of the template  # noqa: E501

        :return: The description of this CreateWorkspaceTemplate.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this CreateWorkspaceTemplate.

        Description of the template  # noqa: E501

        :param description: The description of this CreateWorkspaceTemplate.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def mins_to_complete(self):
        """Gets the mins_to_complete of this CreateWorkspaceTemplate.  # noqa: E501

        Estimated minutes to complete this template  # noqa: E501

        :return: The mins_to_complete of this CreateWorkspaceTemplate.  # noqa: E501
        :rtype: int
        """
        return self._mins_to_complete

    @mins_to_complete.setter
    def mins_to_complete(self, mins_to_complete):
        """Sets the mins_to_complete of this CreateWorkspaceTemplate.

        Estimated minutes to complete this template  # noqa: E501

        :param mins_to_complete: The mins_to_complete of this CreateWorkspaceTemplate.  # noqa: E501
        :type: int
        """

        self._mins_to_complete = mins_to_complete

    @property
    def icon_type(self):
        """Gets the icon_type of this CreateWorkspaceTemplate.  # noqa: E501

        Icon type for the template  # noqa: E501

        :return: The icon_type of this CreateWorkspaceTemplate.  # noqa: E501
        :rtype: WorkspaceTemplateIconType
        """
        return self._icon_type

    @icon_type.setter
    def icon_type(self, icon_type):
        """Sets the icon_type of this CreateWorkspaceTemplate.

        Icon type for the template  # noqa: E501

        :param icon_type: The icon_type of this CreateWorkspaceTemplate.  # noqa: E501
        :type: WorkspaceTemplateIconType
        """

        self._icon_type = icon_type

    @property
    def icon_bg_color(self):
        """Gets the icon_bg_color of this CreateWorkspaceTemplate.  # noqa: E501

        Background color for the icon  # noqa: E501

        :return: The icon_bg_color of this CreateWorkspaceTemplate.  # noqa: E501
        :rtype: WorkspaceTemplateIconBackgroundColor
        """
        return self._icon_bg_color

    @icon_bg_color.setter
    def icon_bg_color(self, icon_bg_color):
        """Sets the icon_bg_color of this CreateWorkspaceTemplate.

        Background color for the icon  # noqa: E501

        :param icon_bg_color: The icon_bg_color of this CreateWorkspaceTemplate.  # noqa: E501
        :type: WorkspaceTemplateIconBackgroundColor
        """

        self._icon_bg_color = icon_bg_color

    @property
    def complexity(self):
        """Gets the complexity of this CreateWorkspaceTemplate.  # noqa: E501

        Complexity level of the template  # noqa: E501

        :return: The complexity of this CreateWorkspaceTemplate.  # noqa: E501
        :rtype: WorkspaceTemplateComplexity
        """
        return self._complexity

    @complexity.setter
    def complexity(self, complexity):
        """Sets the complexity of this CreateWorkspaceTemplate.

        Complexity level of the template  # noqa: E501

        :param complexity: The complexity of this CreateWorkspaceTemplate.  # noqa: E501
        :type: WorkspaceTemplateComplexity
        """

        self._complexity = complexity

    @property
    def category(self):
        """Gets the category of this CreateWorkspaceTemplate.  # noqa: E501

        Category of the template  # noqa: E501

        :return: The category of this CreateWorkspaceTemplate.  # noqa: E501
        :rtype: WorkspaceTemplateCategory
        """
        return self._category

    @category.setter
    def category(self, category):
        """Sets the category of this CreateWorkspaceTemplate.

        Category of the template  # noqa: E501

        :param category: The category of this CreateWorkspaceTemplate.  # noqa: E501
        :type: WorkspaceTemplateCategory
        """

        self._category = category

    @property
    def labels(self):
        """Gets the labels of this CreateWorkspaceTemplate.  # noqa: E501

        Searchable labels for the template  # noqa: E501

        :return: The labels of this CreateWorkspaceTemplate.  # noqa: E501
        :rtype: list[str]
        """
        return self._labels

    @labels.setter
    def labels(self, labels):
        """Sets the labels of this CreateWorkspaceTemplate.

        Searchable labels for the template  # noqa: E501

        :param labels: The labels of this CreateWorkspaceTemplate.  # noqa: E501
        :type: list[str]
        """

        self._labels = labels

    @property
    def published_version_id(self):
        """Gets the published_version_id of this CreateWorkspaceTemplate.  # noqa: E501

        ID of the published version  # noqa: E501

        :return: The published_version_id of this CreateWorkspaceTemplate.  # noqa: E501
        :rtype: str
        """
        return self._published_version_id

    @published_version_id.setter
    def published_version_id(self, published_version_id):
        """Sets the published_version_id of this CreateWorkspaceTemplate.

        ID of the published version  # noqa: E501

        :param published_version_id: The published_version_id of this CreateWorkspaceTemplate.  # noqa: E501
        :type: str
        """

        self._published_version_id = published_version_id

    @property
    def cloud_id(self):
        """Gets the cloud_id of this CreateWorkspaceTemplate.  # noqa: E501

        Cloud ID this template is associated with  # noqa: E501

        :return: The cloud_id of this CreateWorkspaceTemplate.  # noqa: E501
        :rtype: str
        """
        return self._cloud_id

    @cloud_id.setter
    def cloud_id(self, cloud_id):
        """Sets the cloud_id of this CreateWorkspaceTemplate.

        Cloud ID this template is associated with  # noqa: E501

        :param cloud_id: The cloud_id of this CreateWorkspaceTemplate.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and cloud_id is None:  # noqa: E501
            raise ValueError("Invalid value for `cloud_id`, must not be `None`")  # noqa: E501

        self._cloud_id = cloud_id

    @property
    def is_global(self):
        """Gets the is_global of this CreateWorkspaceTemplate.  # noqa: E501

        Whether this is a global template  # noqa: E501

        :return: The is_global of this CreateWorkspaceTemplate.  # noqa: E501
        :rtype: bool
        """
        return self._is_global

    @is_global.setter
    def is_global(self, is_global):
        """Sets the is_global of this CreateWorkspaceTemplate.

        Whether this is a global template  # noqa: E501

        :param is_global: The is_global of this CreateWorkspaceTemplate.  # noqa: E501
        :type: bool
        """

        self._is_global = is_global

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CreateWorkspaceTemplate):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, CreateWorkspaceTemplate):
            return True

        return self.to_dict() != other.to_dict()



import asyncio
from collections import deque
from dataclasses import asdict, fields
from enum import Enum, EnumMeta
import inspect
from typing import (
    Any,
    Awaitable,
    Callable,
    ClassVar,
    Deque,
    Dict,
    Generic,
    Iterable,
    Iterator,
    List,
    Optional,
    Tuple,
    Type,
    TypeVar,
    Union,
)

import yaml


# In Python 3.11 there is a 'Self' introduced, but this is the
# workaround before that: https://peps.python.org/pep-0673/.
T = TypeVar("T")
TModelBase = TypeVar("TModelBase", bound="ModelBase")


class ModelEnumType(EnumMeta):
    def __new__(cls, subcls, bases, classdict):
        new_cls = super().__new__(cls, subcls, bases, classdict)

        # Assert that all enum values have docstrings.
        if not isinstance(new_cls.__docstrings__, dict) or not all(
            isinstance(k, str) and isinstance(v, str)
            for k, v in new_cls.__docstrings__.items()
        ):
            raise TypeError(
                f"ModelEnum '{new_cls.__name__}.__docstrings__' is the wrong type. "
                "Must be a Dict[ModelEnum, str] documenting all enum values"
            )

        missing = {v.value for v in new_cls if not v.name.startswith("_")} - set(
            new_cls.__docstrings__.keys()
        )
        if missing:
            raise ValueError(
                f"ModelEnum '{new_cls.__name__}.__docstrings__' is missing docstrings for values: {sorted(missing)}"
            )

        return new_cls


M = TypeVar("M", bound="ModelEnum")


class ModelEnum(str, Enum, metaclass=ModelEnumType):
    # Must be populated by subclasses to document every value.
    __docstrings__: ClassVar[Dict[str, str]] = {}

    def __str__(self):
        return self.name

    @classmethod
    def validate(cls: Type[M], value: Union[str, M]) -> M:
        allowed_values = [str(s) for s in cls.__members__ if not str(s).startswith("_")]
        if value.upper() in allowed_values:
            return cls(value.upper())
        else:
            raise ValueError(
                f"'{value.upper()}' is not a valid {cls.__name__}. Allowed values are {allowed_values}"
            )


class ModelBaseType(type):
    pass


class ModelBase(metaclass=ModelBaseType):
    def __new__(cls, *_args, **_kwargs):
        """Validate that the subclass is a conforming dataclass."""
        if (
            not hasattr(cls, "__dataclass_params__")
            or not cls.__dataclass_params__.frozen
        ):
            raise TypeError(
                "Subclasses of `ModelBase` must be dataclasses with `frozen=True`."
            )

        return super().__new__(cls)

    def _run_validators(self):
        """Run magic validator methods.

        Looks for methods with the name: `_validate_{field}`. The method will be called
        with the field value as its sole argument.

        If the validator has a return type annotation, the field will be updated to the
        returned value.
        """
        if getattr(self, "__ignore_validation__", False):
            return

        for field in fields(self):
            if not field.metadata.get("docstring", None):
                raise ValueError(
                    f"Model '{type(self).__name__}' is missing docstring for field '{field.name}'."
                )

            validator = getattr(self, f"_validate_{field.name}", None)
            if validator is None:
                raise RuntimeError(
                    f"Model '{type(self).__name__}' is missing validator method for field '{field.name}'."
                )

            signature: inspect.Signature = inspect.signature(validator)
            if len(signature.parameters) != 1:
                raise TypeError(
                    f"Validator for field '{field.name}' must take the field value as its only argument."
                )
            elif field.name not in signature.parameters:
                raise TypeError(
                    f"Validator for field '{field.name}' has invalid parameter name (must match the field name)."
                )
            elif signature.parameters[field.name].annotation == inspect.Parameter.empty:
                raise TypeError(
                    f"Validator for field '{field.name}' is missing a type annotation (should be '{field.type}')."
                )
            elif signature.parameters[field.name].annotation != field.type:
                raise TypeError(
                    f"Validator for field '{field.name}' has the wrong type annotation (should be '{field.type}')."
                )

            maybe_updated_field_value = validator(getattr(self, field.name))
            if signature.return_annotation != inspect.Signature.empty:
                # Only update the value if the validator has a return type annotation.
                object.__setattr__(self, field.name, maybe_updated_field_value)

    def __post_init__(self):
        self._run_validators()

    @classmethod
    def from_dict(cls: Type[TModelBase], d: Dict[str, Any]) -> TModelBase:
        return cls(**d)

    @classmethod
    def from_yaml(cls: Type[TModelBase], path: str, **kwargs) -> TModelBase:
        with open(path) as f:
            args = yaml.safe_load(f) or {}
            if kwargs:
                args.update(kwargs)
            return cls.from_dict(args)

    @classmethod
    def parse_from_internal_model(
        cls: Type[TModelBase], _internal_model: Any
    ) -> TModelBase:
        raise NotImplementedError(
            f"{cls.__name__}.parse_from_internal_model must be implemented"
        )

    def to_dict(self, *, exclude_none: bool = True) -> Dict[str, Any]:
        """Convert the model to a dictionary representation.

        If `exclude_none` is `True`, keys whose values are `None` will be excluded.
        """
        # First, manually convert any nested ModelBase objects to dicts
        # because asdict() would convert them to dicts using raw field values
        converted_fields = {}
        for field in fields(self):
            value = getattr(self, field.name)

            if isinstance(value, ModelBase):
                # Convert nested ModelBase objects using their to_dict method
                converted_fields[field.name] = value.to_dict(exclude_none=exclude_none)
            else:
                converted_fields[field.name] = value

        def maybe_exclude_nones(i: Iterable[Tuple[str, Any]]):
            d = {}
            for k, v in i:
                # Exclude none values if specified.
                if exclude_none and v is None:
                    continue

                # Convert enums to their string representation.
                final_v = v
                if isinstance(v, ModelEnum):
                    final_v = v.value
                elif k in converted_fields and isinstance(converted_fields[k], dict):
                    final_v = converted_fields[k]
                d[k] = final_v

            return d

        return asdict(self, dict_factory=maybe_exclude_nones)

    def options(self: TModelBase, **kwargs) -> TModelBase:
        """Return a copy of the model with the provided fields updated.

        All fields in the constructor are supported. Those not provided will be unchanged.
        """
        new_instance_kwargs = {
            field.name: kwargs.pop(field.name)
            if field.name in kwargs
            else getattr(self, field.name)
            for field in fields(self)
        }
        if len(kwargs) > 0:
            raise ValueError(
                f"Unexpected values passed to '.options': {list(kwargs.keys())}."
            )

        return type(self)(**new_instance_kwargs)


class InternalListResponse(Generic[T]):
    results: List[T]
    has_more: bool


class ListResponse(List[TModelBase]):
    """
    List, but with `has_more` attribute.

    Attributes:
        has_more (bool): Whether there are more results to fetch.

    To use:
    ```
    entities: ListResponse[X] = anyscale.<entity>.list(limit=10)
    for x in entities:
        print(x)
    ```
    """

    def __init__(
        self,
        after: Optional[str],
        limit: Optional[int],
        get_next_page: Callable[[Optional[str]], InternalListResponse[T]],
        cls: Type[TModelBase],
    ):
        super().__init__()
        self.has_more = True

        self._after = after
        self._limit = limit or 1000
        self._get_next_page = get_next_page
        self._cls = cls

        # Fetch the first page
        self._fetch_next_page()

    def _fetch_next_page(self):
        if not self.has_more:
            return
        next_page = self._get_next_page(self._after)
        self.has_more = next_page.has_more
        self.extend([self._cls.parse_from_internal_model(r) for r in next_page.results])
        del self[self._limit :]
        self._after = self[-1].id if self else None  # type: ignore

    def __iter__(self) -> Iterator[TModelBase]:
        index = 0
        while index < len(self):
            yield self[index]
            index += 1
            if index >= self._limit:
                return
            if index >= len(self) and self.has_more:
                self._fetch_next_page()


RT = TypeVar("RT")


class ResultIterator(Generic[RT]):
    """
    Lazily fetch and parse pages from a paged-list API that returns
    Pydantic models with `.results` and `.metadata.next_paging_token`.
    """

    def __init__(
        self,
        *,
        page_token: Optional[str],
        max_items: Optional[int],
        fetch_page: Callable[[Optional[str]], Any],
        parse_fn: Optional[Callable[[Any], RT]] = None,
        async_parse_fn: Optional[Callable[[Any], Awaitable[RT]]] = None,
    ):
        if parse_fn and async_parse_fn:
            raise ValueError("Only one of parse_fn or async_parse_fn may be provided")

        self._token = page_token
        self._max = max_items
        self._fetch = fetch_page
        self._parse = parse_fn
        self._aparse = async_parse_fn
        self._buffer: Deque[RT] = deque()
        self._count = 0
        self._finished = False

    def __iter__(self) -> Iterator[RT]:
        while True:
            # 1) Drain the buffer
            while self._buffer:
                if self._max is not None and self._count >= self._max:
                    return
                self._count += 1
                yield self._buffer.popleft()

            # 2) Done?
            if self._finished or (self._max is not None and self._count >= self._max):
                return

            # 3) Fetch the next page (Pydantic model)
            page = self._fetch(self._token)
            raw_results = page.results
            self._token = page.metadata.next_paging_token

            # 4) No more data?
            if not raw_results:
                self._finished = True
                return

            # 5) Parse—sync or async
            if self._aparse:
                processed = asyncio.run(
                    ResultIterator._process_items_async(raw_results, self._aparse)
                )
                self._buffer.extend(processed)

            elif self._parse:
                try:
                    for raw in raw_results:
                        self._buffer.append(self._parse(raw))
                except Exception as e:  # noqa: BLE001
                    raise RuntimeError(f"sync parse error: {e}") from e

            else:
                # No parser: assume items are already RT
                self._buffer.extend(raw_results)  # type: ignore

            # 6) If no next token, finish on next loop
            if self._token is None:
                self._finished = True

    @staticmethod
    async def _process_items_async(
        items: List[Any], parser: Callable[[Any], Awaitable[RT]],
    ) -> List[RT]:
        if not items:
            return []
        tasks = [parser(item) for item in items]
        results = await asyncio.gather(*tasks, return_exceptions=True)
        processed: List[RT] = []
        for idx, res in enumerate(results):
            if isinstance(res, Exception):
                raise RuntimeError(f"async parse failed on item {idx}: {res}") from res
            processed.append(res)
        return processed



from datetime import date, datetime
from functools import wraps
import sys
from typing import Callable, Dict, Optional, Type, TypeVar, Union

import colorama
from typing_extensions import ParamSpec


P = ParamSpec("P")
T = TypeVar("T")


_LAZY_SDK_SINGLETONS: Dict[str, Callable] = {}


def sdk_command(
    key: str, sdk_cls: Type, *, doc_py_example: str, arg_docstrings: Dict[str, str],
) -> Callable[[Callable[P, T]], Callable[P, T]]:
    """Decorator to automatically inject an `_sdk` arg into the wrapped function.

    The arguments to this class are a unique key for the singleton and its type
    (the constructor will be called with no arguments).
    """

    # The P and T type hints allow f's type hints to pass through this decorator.
    # Without them, f's type hints would not be visible to the developer.
    # See https://github.com/anyscale/product/pull/27738.
    def _inject_typed_sdk_singleton(f: Callable[P, T]) -> Callable[P, T]:
        if not doc_py_example:
            raise ValueError(
                f"SDK command '{f.__name__}' must provide a non-empty 'doc_py_example'."
            )

        # TODO: validate docstrings.

        @wraps(f)
        def wrapper(*args: P.args, **kwargs: P.kwargs) -> T:
            # We disable the mypy linter here because it treats kwargs as a
            # P.kwargs object. mypy wrongly thinks kwargs can't be indexed.
            if "_private_sdk" not in kwargs:  # type: ignore
                if key not in _LAZY_SDK_SINGLETONS:
                    _LAZY_SDK_SINGLETONS[key] = sdk_cls()

                kwargs["_private_sdk"] = _LAZY_SDK_SINGLETONS[key]  # type: ignore

            return f(*args, **kwargs)

        # TODO(edoakes): move to parsing docstrings instead.
        wrapper.__doc_py_example__ = doc_py_example  # type: ignore
        wrapper.__arg_docstrings__ = arg_docstrings  # type: ignore

        return wrapper

    return _inject_typed_sdk_singleton


def sdk_docs(
    *, doc_py_example: str, arg_docstrings: Dict[str, str],
) -> Callable[[Callable[P, T]], Callable[P, T]]:
    """Decorator to add documentation for an SDK command."""

    # The P and T type hints allow f's type hints to pass through this decorator.
    # Without them, f's type hints would not be visible to the developer.
    # See https://github.com/anyscale/product/pull/27738.
    def _add_doc_magic_attrs(f: Callable[P, T]) -> Callable[P, T]:
        if not doc_py_example:
            raise ValueError(
                f"SDK command '{f.__name__}' must provide a non-empty 'doc_py_example'."
            )

        # TODO(edoakes): validate docstrings.
        # TODO(edoakes): move to parsing docstrings instead.
        f.__doc_py_example__ = doc_py_example  # type: ignore
        f.__arg_docstrings__ = arg_docstrings  # type: ignore

        return f

    return _add_doc_magic_attrs


def sdk_command_v2(
    *, doc_py_example: str, arg_docstrings: Dict[str, str],
) -> Callable[[Callable[P, T]], Callable[P, T]]:
    """Similar to `@sdk_command`, but relies on the SDK function initializing `sdk()` in the function body.

    Decorates the function with the provided `doc_py_example` and `arg_docstrings`.
    """

    def _wrap_sdk_function(f: Callable[P, T]) -> Callable[P, T]:
        if not doc_py_example:
            raise ValueError(
                f"SDK command '{f.__name__}' must provide a non-empty 'doc_py_example'."
            )

        @wraps(f)
        def wrapper(*args: P.args, **kwargs: P.kwargs) -> T:
            return f(*args, **kwargs)

        wrapper.__doc_py_example__ = doc_py_example  # type: ignore
        wrapper.__arg_docstrings__ = arg_docstrings  # type: ignore

        return wrapper

    return _wrap_sdk_function


def deprecated_sdk_command(
    key: str,
    sdk_cls: Type,
    *,
    doc_py_example: str,
    arg_docstrings: Dict[str, str],
    deprecation_message: Optional[str] = None,
    removal_date: Optional[Union[str, date, datetime]] = None,
    alternative: Optional[str] = None,
) -> Callable[[Callable[P, T]], Callable[P, T]]:
    """SDK command decorator with deprecation warning support.

    Similar to sdk_command but adds deprecation warnings when the function is called.

    Args:
        key: Unique key for the SDK singleton
        sdk_cls: SDK class type
        doc_py_example: Documentation example
        arg_docstrings: Argument documentation
        deprecation_message: Custom deprecation message
        removal_date: When the command will be removed (YYYY-MM-DD string, date, or datetime)
        alternative: Suggested alternative command
    """

    def _inject_typed_sdk_singleton_with_deprecation(
        f: Callable[P, T]
    ) -> Callable[P, T]:
        if not doc_py_example:
            raise ValueError(
                f"SDK command '{f.__name__}' must provide a non-empty 'doc_py_example'."
            )

        @wraps(f)
        def wrapper(*args: P.args, **kwargs: P.kwargs) -> T:  # noqa: PLR0912
            # Print a visual separator for attention
            print("\n" + "=" * 80, file=sys.stderr)
            print(
                f"{colorama.Style.BRIGHT}{colorama.Fore.YELLOW}⚠️  SDK DEPRECATION WARNING ⚠️{colorama.Style.RESET_ALL}",
                file=sys.stderr,
            )
            print("=" * 80, file=sys.stderr)

            # Build deprecation message
            base_msg = deprecation_message or f"Function '{f.__name__}' is deprecated"

            # Add removal date information with grammar-aware connector
            date_msg = None
            if removal_date:
                date_str = None
                try:
                    if isinstance(removal_date, str):
                        parsed_date = datetime.strptime(removal_date, "%Y-%m-%d").date()
                    elif isinstance(removal_date, datetime):
                        parsed_date = removal_date.date()
                    elif isinstance(removal_date, date):
                        parsed_date = removal_date
                    else:
                        parsed_date = None

                    if parsed_date:
                        date_str = parsed_date.strftime("%Y-%m-%d")
                except (ValueError, AttributeError):
                    date_str = str(removal_date)

                if date_str:
                    ends_with_punct = base_msg.strip().endswith((".", "!", "?"))
                    if ends_with_punct:
                        date_msg = f"It will be removed on {date_str}"
                    else:
                        date_msg = f"and will be removed on {date_str}"

            # Add alternative suggestion
            alternative_msg = None
            if alternative:
                alternative_msg = f"\n\n➡️  {colorama.Style.BRIGHT}Please {alternative}{colorama.Style.RESET_ALL}"

            msg_parts = {
                "deprecation_message": base_msg,
                "date_msg": date_msg,
                "alternative_msg": alternative_msg,
            }

            # Join the main line with a space, then append the alternative on its own line
            main_line_parts = [
                part
                for part in [msg_parts["deprecation_message"], msg_parts["date_msg"]]
                if part
            ]
            deprecation_msg = " ".join(main_line_parts)
            if msg_parts["alternative_msg"]:
                deprecation_msg += msg_parts["alternative_msg"]

            # Print the warning
            print(
                f"\n{colorama.Fore.YELLOW}{deprecation_msg}{colorama.Style.RESET_ALL}",
                file=sys.stderr,
            )
            print("=" * 80 + "\n", file=sys.stderr)

            # Then handle SDK injection like normal sdk_command
            if "_private_sdk" not in kwargs:  # type: ignore
                if key not in _LAZY_SDK_SINGLETONS:
                    _LAZY_SDK_SINGLETONS[key] = sdk_cls()

                kwargs["_private_sdk"] = _LAZY_SDK_SINGLETONS[key]  # type: ignore

            return f(*args, **kwargs)

        wrapper.__doc_py_example__ = doc_py_example  # type: ignore
        wrapper.__arg_docstrings__ = arg_docstrings  # type: ignore
        wrapper.__deprecated__ = True  # type: ignore
        wrapper.__deprecation_message__ = deprecation_message  # type: ignore
        wrapper.__removal_date__ = removal_date  # type: ignore
        wrapper.__alternative__ = alternative  # type: ignore

        return wrapper

    return _inject_typed_sdk_singleton_with_deprecation



from typing import List, Optional

from anyscale._private.models.model_base import ResultIterator
from anyscale._private.sdk.base_sdk import BaseSDK
from anyscale.cli_logger import BlockLogger
from anyscale.client.openapi_client.models import (
    Cloud as CloudModel,
    CreateCloudCollaborator as CreateCloudCollaboratorModel,
)
from anyscale.cloud.models import (
    Cloud,
    CloudProvider,
    ComputeStack,
    CreateCloudCollaborator,
)
from anyscale.sdk.anyscale_client.models import ClusterState


logger = BlockLogger()


class PrivateCloudSDK(BaseSDK):
    def add_collaborators(
        self, cloud: str, collaborators: List[CreateCloudCollaborator]
    ) -> None:
        cloud_id = self.client.get_cloud_id(cloud_name=cloud, compute_config_id=None)

        self.client.add_cloud_collaborators(
            cloud_id=cloud_id,
            collaborators=[
                CreateCloudCollaboratorModel(
                    email=collaborator.email,
                    permission_level=collaborator.permission_level.lower(),
                )
                for collaborator in collaborators
            ],
        )

    def get(
        self, id: Optional[str], name: Optional[str],  # noqa: A002
    ) -> Optional[Cloud]:
        if (id and name) or (not id and not name):
            raise ValueError("Provide exactly one of 'id' or 'name'.")

        if id:
            openapi_cloud = self.client.get_cloud(cloud_id=id)
        else:
            assert name is not None, "Name must be provided if id is not."
            openapi_cloud = self.client.get_cloud_by_name(name=name)

        return self._to_sdk_cloud(openapi_cloud)

    def get_default(self) -> Optional[Cloud]:
        openapi_cloud = self.client.get_default_cloud()

        return self._to_sdk_cloud(openapi_cloud)

    def list(
        self,
        *,
        cloud_id: Optional[str] = None,
        name: Optional[str] = None,
        max_items: Optional[int] = None,
        page_size: Optional[int] = None,
    ):
        # Single item by ID
        if cloud_id is not None:
            openapi_cloud = self.client.get_cloud(cloud_id=cloud_id)
            cloud = self._to_sdk_cloud(openapi_cloud)
            return [cloud] if cloud is not None else []

        # Single item by name
        if name is not None:
            openapi_cloud = self.client.get_cloud_by_name(name=name)
            cloud = self._to_sdk_cloud(openapi_cloud)
            return [cloud] if cloud is not None else []

        # Iterator over all
        def _fetch(token: Optional[str]):
            return self.client.list_clouds(paging_token=token, count=page_size)

        return ResultIterator[Cloud](
            page_token=None,
            max_items=max_items,
            fetch_page=_fetch,
            parse_fn=self._to_sdk_cloud,
        )

    def _to_sdk_cloud(self, openapi_cloud: Optional["CloudModel"]) -> Optional[Cloud]:
        if openapi_cloud is None:
            return None

        # Validate provider, default to UNKNOWN if validation fails
        if openapi_cloud.provider is not None:
            try:
                provider = CloudProvider.validate(openapi_cloud.provider)
            except ValueError:
                provider = CloudProvider.UNKNOWN
        else:
            provider = CloudProvider.UNKNOWN

        # Validate compute_stack, default to UNKNOWN if validation fails
        if openapi_cloud.compute_stack is not None:
            try:
                compute_stack = ComputeStack.validate(openapi_cloud.compute_stack)
            except ValueError:
                compute_stack = ComputeStack.UNKNOWN
        else:
            compute_stack = ComputeStack.UNKNOWN

        return Cloud(
            id=openapi_cloud.id,
            name=openapi_cloud.name,
            provider=provider,
            region=openapi_cloud.region,
            created_at=openapi_cloud.created_at,
            is_default=openapi_cloud.is_default,
            compute_stack=compute_stack,
        )

    def terminate_system_cluster(self, cloud_id: str, wait: bool) -> str:
        resp = self.client.terminate_system_cluster(cloud_id)
        if wait:
            self._wait_for_system_cluster_status(cloud_id, ClusterState.TERMINATED)
        else:
            logger.info(f"System cluster termination initiated for cloud {cloud_id}.")
        return resp.result.cluster_id

    def _wait_for_system_cluster_status(
        self,
        cloud_id: str,
        goal_status: str,
        timeout_s: int = 500,
        interval_s: int = 10,
    ) -> bool:
        self.logger.info("Waiting for system cluster termination...", end="")
        for _ in self.timer.poll(timeout_s=timeout_s, interval_s=interval_s):
            status = self.client.describe_system_workload_get_status(cloud_id)
            if status == goal_status:
                print(".")
                self.logger.info(f"System cluster for cloud '{cloud_id}' is {status}.")
                return True
            else:
                print(".", end="")
        raise TimeoutError(
            f"Timed out waiting for system cluster termination for cloud '{cloud_id}'. Last seen status: {status}."
        )



from typing import List, Optional

from anyscale._private.anyscale_client import AnyscaleClientInterface
from anyscale._private.models.model_base import ResultIterator
from anyscale._private.sdk import sdk_docs
from anyscale._private.sdk.base_sdk import Timer
from anyscale.cli_logger import BlockLogger
from anyscale.user._private.user_sdk import PrivateUserSDK
from anyscale.user.commands import (
    _ADMIN_BATCH_CREATE_ARG_DOCSTRINGS,
    _ADMIN_BATCH_CREATE_EXAMPLE,
    _GET_ARG_DOCSTRINGS,
    _GET_EXAMPLE,
    _LIST_ARG_DOCSTRINGS,
    _LIST_EXAMPLE,
    admin_batch_create as admin_batch_create,
    get as get,
    list as list,  # noqa: A004 - claude_comment("claude-opus-4-5", "SDK public API re-export")
)
from anyscale.user.models import AdminCreatedUser, AdminCreateUser, User


class UserSDK:
    def __init__(
        self,
        *,
        client: Optional[AnyscaleClientInterface] = None,
        logger: Optional[BlockLogger] = None,
        timer: Optional[Timer] = None,
    ):
        self._private_sdk = PrivateUserSDK(client=client, logger=logger, timer=timer)

    @sdk_docs(
        doc_py_example=_ADMIN_BATCH_CREATE_EXAMPLE,
        arg_docstrings=_ADMIN_BATCH_CREATE_ARG_DOCSTRINGS,
    )
    def admin_batch_create(  # noqa: F811
        self, admin_create_users: List[AdminCreateUser],
    ) -> List[AdminCreatedUser]:
        """Batch create, as an admin, users without email verification.
        """
        return self._private_sdk.admin_batch_create(admin_create_users)

    @sdk_docs(
        doc_py_example=_LIST_EXAMPLE, arg_docstrings=_LIST_ARG_DOCSTRINGS,
    )
    def list(  # noqa: F811
        self,
        *,
        email: Optional[str] = None,
        name: Optional[str] = None,
        collaborator_type: Optional[str] = None,
        is_service_account: Optional[bool] = None,
        max_items: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> ResultIterator[User]:
        """List organization collaborators."""
        return self._private_sdk.list(
            email=email,
            name=name,
            collaborator_type=collaborator_type,
            is_service_account=is_service_account,
            max_items=max_items,
            page_size=page_size,
        )

    @sdk_docs(
        doc_py_example=_GET_EXAMPLE, arg_docstrings=_GET_ARG_DOCSTRINGS,
    )
    def get(  # noqa: F811
        self,
        *,
        email: Optional[str] = None,
        name: Optional[str] = None,
        collaborator_type: Optional[str] = None,
        is_service_account: Optional[bool] = None,
    ) -> User:
        """Retrieve a single collaborator by email or name."""
        return self._private_sdk.get(
            email=email,
            name=name,
            collaborator_type=collaborator_type,
            is_service_account=is_service_account,
        )



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class AWSConfig(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'vpc_id': 'str',
        'subnet_ids': 'list[str]',
        'zones': 'list[str]',
        'security_group_ids': 'list[str]',
        'anyscale_iam_role_id': 'str',
        'external_id': 'str',
        'cluster_iam_role_id': 'str',
        'memorydb_cluster_name': 'str',
        'memorydb_cluster_arn': 'str',
        'memorydb_cluster_endpoint': 'str',
        'cloudformation_id': 'str'
    }

    attribute_map = {
        'vpc_id': 'vpc_id',
        'subnet_ids': 'subnet_ids',
        'zones': 'zones',
        'security_group_ids': 'security_group_ids',
        'anyscale_iam_role_id': 'anyscale_iam_role_id',
        'external_id': 'external_id',
        'cluster_iam_role_id': 'cluster_iam_role_id',
        'memorydb_cluster_name': 'memorydb_cluster_name',
        'memorydb_cluster_arn': 'memorydb_cluster_arn',
        'memorydb_cluster_endpoint': 'memorydb_cluster_endpoint',
        'cloudformation_id': 'cloudformation_id'
    }

    def __init__(self, vpc_id=None, subnet_ids=None, zones=None, security_group_ids=None, anyscale_iam_role_id=None, external_id=None, cluster_iam_role_id=None, memorydb_cluster_name=None, memorydb_cluster_arn=None, memorydb_cluster_endpoint=None, cloudformation_id=None, local_vars_configuration=None):  # noqa: E501
        """AWSConfig - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._vpc_id = None
        self._subnet_ids = None
        self._zones = None
        self._security_group_ids = None
        self._anyscale_iam_role_id = None
        self._external_id = None
        self._cluster_iam_role_id = None
        self._memorydb_cluster_name = None
        self._memorydb_cluster_arn = None
        self._memorydb_cluster_endpoint = None
        self._cloudformation_id = None
        self.discriminator = None

        if vpc_id is not None:
            self.vpc_id = vpc_id
        if subnet_ids is not None:
            self.subnet_ids = subnet_ids
        if zones is not None:
            self.zones = zones
        if security_group_ids is not None:
            self.security_group_ids = security_group_ids
        if anyscale_iam_role_id is not None:
            self.anyscale_iam_role_id = anyscale_iam_role_id
        if external_id is not None:
            self.external_id = external_id
        if cluster_iam_role_id is not None:
            self.cluster_iam_role_id = cluster_iam_role_id
        if memorydb_cluster_name is not None:
            self.memorydb_cluster_name = memorydb_cluster_name
        if memorydb_cluster_arn is not None:
            self.memorydb_cluster_arn = memorydb_cluster_arn
        if memorydb_cluster_endpoint is not None:
            self.memorydb_cluster_endpoint = memorydb_cluster_endpoint
        if cloudformation_id is not None:
            self.cloudformation_id = cloudformation_id

    @property
    def vpc_id(self):
        """Gets the vpc_id of this AWSConfig.  # noqa: E501

        The VPC ID.  # noqa: E501

        :return: The vpc_id of this AWSConfig.  # noqa: E501
        :rtype: str
        """
        return self._vpc_id

    @vpc_id.setter
    def vpc_id(self, vpc_id):
        """Sets the vpc_id of this AWSConfig.

        The VPC ID.  # noqa: E501

        :param vpc_id: The vpc_id of this AWSConfig.  # noqa: E501
        :type: str
        """

        self._vpc_id = vpc_id

    @property
    def subnet_ids(self):
        """Gets the subnet_ids of this AWSConfig.  # noqa: E501

        List of subnet IDs.  # noqa: E501

        :return: The subnet_ids of this AWSConfig.  # noqa: E501
        :rtype: list[str]
        """
        return self._subnet_ids

    @subnet_ids.setter
    def subnet_ids(self, subnet_ids):
        """Sets the subnet_ids of this AWSConfig.

        List of subnet IDs.  # noqa: E501

        :param subnet_ids: The subnet_ids of this AWSConfig.  # noqa: E501
        :type: list[str]
        """

        self._subnet_ids = subnet_ids

    @property
    def zones(self):
        """Gets the zones of this AWSConfig.  # noqa: E501

        The availability zone corresponding to each subnet ID.  # noqa: E501

        :return: The zones of this AWSConfig.  # noqa: E501
        :rtype: list[str]
        """
        return self._zones

    @zones.setter
    def zones(self, zones):
        """Sets the zones of this AWSConfig.

        The availability zone corresponding to each subnet ID.  # noqa: E501

        :param zones: The zones of this AWSConfig.  # noqa: E501
        :type: list[str]
        """

        self._zones = zones

    @property
    def security_group_ids(self):
        """Gets the security_group_ids of this AWSConfig.  # noqa: E501

        List of security group IDs.  # noqa: E501

        :return: The security_group_ids of this AWSConfig.  # noqa: E501
        :rtype: list[str]
        """
        return self._security_group_ids

    @security_group_ids.setter
    def security_group_ids(self, security_group_ids):
        """Sets the security_group_ids of this AWSConfig.

        List of security group IDs.  # noqa: E501

        :param security_group_ids: The security_group_ids of this AWSConfig.  # noqa: E501
        :type: list[str]
        """

        self._security_group_ids = security_group_ids

    @property
    def anyscale_iam_role_id(self):
        """Gets the anyscale_iam_role_id of this AWSConfig.  # noqa: E501

        The Anyscale IAM role ARN.  # noqa: E501

        :return: The anyscale_iam_role_id of this AWSConfig.  # noqa: E501
        :rtype: str
        """
        return self._anyscale_iam_role_id

    @anyscale_iam_role_id.setter
    def anyscale_iam_role_id(self, anyscale_iam_role_id):
        """Sets the anyscale_iam_role_id of this AWSConfig.

        The Anyscale IAM role ARN.  # noqa: E501

        :param anyscale_iam_role_id: The anyscale_iam_role_id of this AWSConfig.  # noqa: E501
        :type: str
        """

        self._anyscale_iam_role_id = anyscale_iam_role_id

    @property
    def external_id(self):
        """Gets the external_id of this AWSConfig.  # noqa: E501

        The trust policy external ID for the cross-account IAM role  # noqa: E501

        :return: The external_id of this AWSConfig.  # noqa: E501
        :rtype: str
        """
        return self._external_id

    @external_id.setter
    def external_id(self, external_id):
        """Sets the external_id of this AWSConfig.

        The trust policy external ID for the cross-account IAM role  # noqa: E501

        :param external_id: The external_id of this AWSConfig.  # noqa: E501
        :type: str
        """

        self._external_id = external_id

    @property
    def cluster_iam_role_id(self):
        """Gets the cluster_iam_role_id of this AWSConfig.  # noqa: E501

        The IAM role ARN used by Ray clusters.  # noqa: E501

        :return: The cluster_iam_role_id of this AWSConfig.  # noqa: E501
        :rtype: str
        """
        return self._cluster_iam_role_id

    @cluster_iam_role_id.setter
    def cluster_iam_role_id(self, cluster_iam_role_id):
        """Sets the cluster_iam_role_id of this AWSConfig.

        The IAM role ARN used by Ray clusters.  # noqa: E501

        :param cluster_iam_role_id: The cluster_iam_role_id of this AWSConfig.  # noqa: E501
        :type: str
        """

        self._cluster_iam_role_id = cluster_iam_role_id

    @property
    def memorydb_cluster_name(self):
        """Gets the memorydb_cluster_name of this AWSConfig.  # noqa: E501

        The MemoryDB cluster name.  # noqa: E501

        :return: The memorydb_cluster_name of this AWSConfig.  # noqa: E501
        :rtype: str
        """
        return self._memorydb_cluster_name

    @memorydb_cluster_name.setter
    def memorydb_cluster_name(self, memorydb_cluster_name):
        """Sets the memorydb_cluster_name of this AWSConfig.

        The MemoryDB cluster name.  # noqa: E501

        :param memorydb_cluster_name: The memorydb_cluster_name of this AWSConfig.  # noqa: E501
        :type: str
        """

        self._memorydb_cluster_name = memorydb_cluster_name

    @property
    def memorydb_cluster_arn(self):
        """Gets the memorydb_cluster_arn of this AWSConfig.  # noqa: E501

        The MemoryDB cluster ARN.  # noqa: E501

        :return: The memorydb_cluster_arn of this AWSConfig.  # noqa: E501
        :rtype: str
        """
        return self._memorydb_cluster_arn

    @memorydb_cluster_arn.setter
    def memorydb_cluster_arn(self, memorydb_cluster_arn):
        """Sets the memorydb_cluster_arn of this AWSConfig.

        The MemoryDB cluster ARN.  # noqa: E501

        :param memorydb_cluster_arn: The memorydb_cluster_arn of this AWSConfig.  # noqa: E501
        :type: str
        """

        self._memorydb_cluster_arn = memorydb_cluster_arn

    @property
    def memorydb_cluster_endpoint(self):
        """Gets the memorydb_cluster_endpoint of this AWSConfig.  # noqa: E501

        The MemoryDB cluster endpoint.  # noqa: E501

        :return: The memorydb_cluster_endpoint of this AWSConfig.  # noqa: E501
        :rtype: str
        """
        return self._memorydb_cluster_endpoint

    @memorydb_cluster_endpoint.setter
    def memorydb_cluster_endpoint(self, memorydb_cluster_endpoint):
        """Sets the memorydb_cluster_endpoint of this AWSConfig.

        The MemoryDB cluster endpoint.  # noqa: E501

        :param memorydb_cluster_endpoint: The memorydb_cluster_endpoint of this AWSConfig.  # noqa: E501
        :type: str
        """

        self._memorydb_cluster_endpoint = memorydb_cluster_endpoint

    @property
    def cloudformation_id(self):
        """Gets the cloudformation_id of this AWSConfig.  # noqa: E501

        The CloudFormation stack ID, for Anyscale-managed resources.  # noqa: E501

        :return: The cloudformation_id of this AWSConfig.  # noqa: E501
        :rtype: str
        """
        return self._cloudformation_id

    @cloudformation_id.setter
    def cloudformation_id(self, cloudformation_id):
        """Sets the cloudformation_id of this AWSConfig.

        The CloudFormation stack ID, for Anyscale-managed resources.  # noqa: E501

        :param cloudformation_id: The cloudformation_id of this AWSConfig.  # noqa: E501
        :type: str
        """

        self._cloudformation_id = cloudformation_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, AWSConfig):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, AWSConfig):
            return True

        return self.to_dict() != other.to_dict()



import json
from typing import Any, Dict, IO, List, Optional, Union

import click
from click import ClickException
import tabulate
import yaml
from yaml.loader import SafeLoader

from anyscale.anyscale_pydantic import (
    BaseModel,
    Field,
    root_validator,
    validator,
)
from anyscale.cli_logger import BlockLogger
from anyscale.cloud_utils import get_cloud_id_and_name
from anyscale.cluster_compute import get_cluster_compute_from_name
from anyscale.conf import IDLE_TIMEOUT_DEFAULT_MINUTES
from anyscale.controllers.base_controller import BaseController
from anyscale.sdk.anyscale_client.models.cluster_compute_config import (
    ClusterComputeConfig,
)
from anyscale.sdk.anyscale_client.models.create_cluster_compute import (
    CreateClusterCompute,
)
from anyscale.sdk.anyscale_client.models.create_cluster_compute_config import (
    CreateClusterComputeConfig,
)
from anyscale.util import get_endpoint
from anyscale.utils.entity_arg_utils import EntityType, IdBasedEntity, NameBasedEntity
from anyscale.utils.name_utils import gen_valid_name


log = BlockLogger()  # Anyscale CLI Logger


class CreateClusterComputeConfigModel(BaseModel):
    """
    Schema for ClusterClusterCompute that is accepted from the CLI. Supports specifying
    `cloud` name instead of `cloud_id`, and preprocesses so fields have the correct
    values to call POST `/ext/v0/cluster_computes`.
    """

    cloud_id: Optional[str] = Field(
        None, description="The ID of the Anyscale cloud to use for launching Clusters.",
    )
    cloud: Optional[str] = Field(
        None,
        description="The name of the Anyscale cloud to use for launching Clusters.",
    )
    max_workers: Optional[int] = Field(
        None, description="Desired limit on total running workers for this Cluster."
    )

    allowed_azs: Optional[List[str]] = Field(
        None,
        description='The availability zones that clusters are allowed to be launched in, e.g. "us-west-2a". If not specified, any AZ may be used.',
    )

    head_node_type: Any = Field(
        ..., description="Node configuration to use for the head node. ",
    )

    worker_node_types: List[Any] = Field(
        ..., description="A list of node types to use for worker nodes. "
    )

    aws: Optional[Any] = Field(
        None,
        description="Fields specific to AWS node types.",
        alias="aws_advanced_configurations_json",
    )

    gcp: Optional[Any] = Field(
        None,
        description="Fields specific to GCP node types.",
        alias="gcp_advanced_configurations_json",
    )

    azure: Optional[Any] = Field(
        None, description="Fields specific to Azure node types.",
    )

    maximum_uptime_minutes: Optional[int] = Field(
        None,
        description="If set to a positive number, Anyscale will terminate the cluster this many minutes after cluster start.",
    )

    idle_termination_minutes: int = Field(
        IDLE_TIMEOUT_DEFAULT_MINUTES,
        description=(
            "If set to a positive number, Anyscale will terminate the cluster this many minutes after the cluster is idle. "
            "Idle time is defined as the time during which a Cluster is not running a user command. "
            "Time spent running commands on Jupyter or ssh is still considered 'idle'. "
            "To disable, set this field to 0."
        ),
        ge=0,
    )

    auto_select_worker_config: Optional[bool] = Field(
        False,
        description="If set to true, worker node groups will automatically be selected based on workload.",
    )

    flags: Optional[Dict[str, Any]] = Field(
        default_factory=dict,
        description="A set of advanced flags that can be used to configure a particular workload.",
    )

    @validator("allowed_azs")
    def check_any_option(cls, v: Optional[List[str]]):
        if not v:
            return v
        if "any" in v and len(v) > 1:
            raise ValueError("Cannot specify 'any' and other AZs.")
        return v

    @root_validator
    def fill_cloud_id(cls: Any, values: Any) -> Any:
        cloud_id, cloud = (
            values.get("cloud_id"),
            values.get("cloud"),
        )
        if cloud_id and cloud:
            raise click.ClickException(
                "Only one of `cloud_id` or `cloud` can be provided in the cluster compute config file. "
            )
        if cloud:
            cloud_id, _ = get_cloud_id_and_name(
                api_client=None, cloud_id=None, cloud_name=cloud
            )
            values["cloud_id"] = cloud_id
        elif not cloud_id:
            raise click.ClickException(
                "Please provide either `cloud_id` or `cloud` in the cluster compute config file. "
            )
        return values

    class Config:
        # This setting will cause pydantic (and fastAPI) to accept the field name as well as the alias
        # (eg. "cpu" or "CPU"). But note that openapi does not respect this setting.
        # Generated models will require the alias.
        allow_population_by_field_name = True


class ComputeConfigController(BaseController):
    """
    This controller powers functionalities related to Anyscale
    cluster compute configuration.
    """

    def __init__(
        self, log: Optional[BlockLogger] = None, initialize_auth_api_client: bool = True
    ):
        if log is None:
            log = BlockLogger()

        super().__init__(initialize_auth_api_client=initialize_auth_api_client)
        self.log = log
        self.log.open_block("Output")

    def create(self, cluster_compute_file: IO[bytes], name: Optional[str]) -> None:
        """Builds a new cluster compute template
        If name is not provided, a default cluster-compute-name will be used and returned in the command output

        Information in output: Link to cluster compute in UI, cluster compute id
        """
        try:
            cluster_compute: Dict[str, Any] = yaml.load(
                cluster_compute_file, Loader=SafeLoader
            )
        except Exception as e:  # noqa: BLE001
            raise ClickException(f"Could not load compute config file: {e}")

        cluster_compute_config_model = CreateClusterComputeConfigModel(
            **cluster_compute
        )
        if cluster_compute_config_model.max_workers is not None:
            log.warning(
                "Warning: global `max_workers` is deprecated for Anyscale Ray 2.7+. Please use global resource max instead: https://docs.anyscale.com/configure/compute-configs/global-resource-min-max"
            )
        cluster_compute_config = CreateClusterComputeConfig(
            cloud_id=cluster_compute_config_model.cloud_id,
            max_workers=cluster_compute_config_model.max_workers,
            allowed_azs=cluster_compute_config_model.allowed_azs,
            region=None,
            head_node_type=cluster_compute_config_model.head_node_type,
            worker_node_types=cluster_compute_config_model.worker_node_types,
            aws_advanced_configurations_json=cluster_compute_config_model.aws,
            gcp_advanced_configurations_json=cluster_compute_config_model.gcp,
            maximum_uptime_minutes=cluster_compute_config_model.maximum_uptime_minutes,
            idle_termination_minutes=cluster_compute_config_model.idle_termination_minutes,
            auto_select_worker_config=cluster_compute_config_model.auto_select_worker_config,
            flags=cluster_compute_config_model.flags,
        )
        if name is None:
            name = gen_valid_name("cli-config")

        cluster_compute_response = self.anyscale_api_client.create_cluster_compute(
            CreateClusterCompute(
                name=name, config=cluster_compute_config, new_version=True,
            )
        )
        created_cluster_compute = cluster_compute_response.result
        cluster_compute_id = created_cluster_compute.id
        cluster_compute_name = created_cluster_compute.name
        cluster_compute_version = created_cluster_compute.version
        url = get_endpoint(f"/configurations/cluster-computes/{cluster_compute_id}")
        if cluster_compute_version > 1:
            log.info(f"A new version of {cluster_compute_name} was created.")
        else:
            log.info("A new compute config was created.")

        log.info(f"View this compute config at: {url}.")
        log.info(f"Compute config id: {cluster_compute_id}.")
        log.info(f"Compute config name: {cluster_compute_name}.")
        log.info(f"Compute config version: {cluster_compute_version}.")

    def archive(
        self, compute_config_entity: Union[IdBasedEntity, NameBasedEntity]
    ) -> None:
        """
        Archives the cluster compute with the given name or id.
        Exactly one of cluster_compute_name or id must be provided.
        """
        if compute_config_entity.type is EntityType.ID:
            compute_config = self.anyscale_api_client.get_compute_template(
                compute_config_entity.id
            ).result
            compute_config_id = compute_config_entity.id
            compute_config_name = compute_config.name
        else:
            compute_config = get_cluster_compute_from_name(
                compute_config_entity.name, self.api_client,
            )
            compute_config_id = compute_config.id
            compute_config_name = compute_config.name
        self.api_client.archive_compute_template_api_v2_compute_templates_compute_template_id_archive_post(
            compute_config_id
        )
        log.info(f"Successfully archived compute config: {compute_config_name}.")

    def list(  # noqa: PLR0913
        self,
        cluster_compute_name: Optional[str],
        cluster_compute_id: Optional[str],
        include_shared: bool,
        max_items: Optional[int] = 20,
        next_token: Optional[str] = None,
        cloud_id: Optional[str] = None,
        cloud_name: Optional[str] = None,
        sort_by: str = "last_modified_at",
        sort_order: str = "asc",
        output_json: bool = False,
    ) -> None:
        """List cluster compute configurations with filtering, sorting, and pagination.

        Args:
            cluster_compute_name: Filter by compute config name
            cluster_compute_id: Filter by specific compute config ID
            include_shared: Include configs shared with the user
            max_items: Maximum number of items to return
            next_token: Pagination token for fetching next page
            cloud_id: Filter by cloud ID
            cloud_name: Filter by cloud name (will be resolved to cloud_id)
            sort_by: Field to sort by (name, created_at, last_modified_at)
            sort_order: Sort order (asc or desc)
            output_json: Output results in JSON format
        """
        # Resolve cloud_name to cloud_id if provided
        resolved_cloud_id = cloud_id
        if cloud_name:
            resolved_cloud_id, cloud_name = get_cloud_id_and_name(
                api_client=self.api_client, cloud_id=None, cloud_name=cloud_name
            )

        cluster_compute_list = []
        final_next_token = None

        if cluster_compute_id:
            # Fetch single compute config by ID
            cluster_compute_list = [
                self.anyscale_api_client.get_cluster_compute(cluster_compute_id).result
            ]
        else:
            # Build query with all applicable filters
            query: Dict[str, Any] = {"paging": {"count": max_items}}

            # Add name filter if specified
            if cluster_compute_name:
                query["name"] = {"equals": cluster_compute_name}

            # Add creator_id filter if not including shared configs
            if not include_shared and not cluster_compute_name:
                creator_id = (
                    self.api_client.get_user_info_api_v2_userinfo_get().result.id
                )
                query["creator_id"] = creator_id

            # Add cloud filter if specified
            if resolved_cloud_id:
                query["cloud_id"] = resolved_cloud_id

            # Add pagination token if provided
            if next_token:
                query["paging"]["paging_token"] = next_token

            # SERVER-SIDE SORTING
            # Map CLI sort parameters to API sort_by_clauses format
            sort_field_map = {
                "name": "NAME",
                "created_at": "CREATED_AT",
                "last_modified_at": "LAST_MODIFIED_AT",
            }
            query["sort_by_clauses"] = [
                {
                    "sort_field": sort_field_map.get(sort_by, "LAST_MODIFIED_AT"),
                    "sort_order": sort_order.upper(),
                }
            ]

            # Make single API call with simplified pagination and server-side sorting
            resp = self.anyscale_api_client.search_cluster_computes(query)
            cluster_compute_list = resp.results
            final_next_token = resp.metadata.next_paging_token

        # Output in JSON format if requested
        if output_json:
            output_data = {
                "results": [
                    {
                        "id": cc.id,
                        "name": cc.name,
                        "cloud_id": cc.config.cloud_id if cc.config else None,
                        "version": cc.version,
                        "created_at": cc.created_at.isoformat()
                        if cc.created_at
                        else None,
                        "last_modified_at": cc.last_modified_at.isoformat()
                        if cc.last_modified_at
                        else None,
                        "url": get_endpoint(f"configurations/cluster-computes/{cc.id}"),
                    }
                    for cc in cluster_compute_list
                ],
                "metadata": {
                    "count": len(cluster_compute_list),
                    "next_token": final_next_token,
                },
            }
            print(json.dumps(output_data, indent=2))
            return

        # Build table for display
        cluster_compute_table = [
            [
                cluster_compute.id,
                cluster_compute.name,
                self.anyscale_api_client.get_cloud(
                    cluster_compute.config.cloud_id
                ).result.name
                if cluster_compute.config.cloud_id
                else None,
                cluster_compute.last_modified_at.strftime("%m/%d/%Y, %H:%M:%S"),
                get_endpoint(f"configurations/cluster-computes/{cluster_compute.id}"),
            ]
            for cluster_compute in cluster_compute_list
        ]

        table = tabulate.tabulate(
            cluster_compute_table,
            headers=["ID", "NAME", "CLOUD", "LAST MODIFIED AT", "URL"],
            tablefmt="plain",
        )
        print(f"Compute configs:\n{table}")

        # Print pagination info if there are more results
        if final_next_token:
            print(
                f"\nMore results available. Use --next-token '{final_next_token}' to fetch the next page."
            )

    def get(
        self,
        cluster_compute_name: Optional[str],
        cluster_compute_id: Optional[str],
        include_archived: Optional[bool] = False,
        cloud_id: Optional[str] = None,
        cloud_name: Optional[str] = None,
    ) -> None:
        """Get details of a specific cluster compute configuration.

        Args:
            cluster_compute_name: Name of the compute config
            cluster_compute_id: ID of the compute config
            include_archived: Include archived compute configs
            cloud_id: Filter by cloud ID when resolving by name
            cloud_name: Filter by cloud name when resolving by name
        """
        if (
            int(cluster_compute_name is not None) + int(cluster_compute_id is not None)
            != 1
        ):
            raise click.ClickException(
                "Please only provide one of `compute-config-name` or `--id`."
            )

        if cluster_compute_name:
            # Resolve cloud_name to cloud_id if provided
            resolved_cloud_name = cloud_name
            if cloud_id:
                # Get cloud name from cloud_id for consistency
                cloud_id, resolved_cloud_name = get_cloud_id_and_name(
                    api_client=self.api_client, cloud_id=cloud_id, cloud_name=None
                )

            # Use cloud_name parameter in get_cluster_compute_from_name
            cluster_compute_id = get_cluster_compute_from_name(
                cluster_compute_name,
                self.api_client,
                include_archived=include_archived,
                cloud_name=resolved_cloud_name,
            ).id

        compute_config: ClusterComputeConfig = self.anyscale_api_client.get_cluster_compute(
            cluster_compute_id
        ).result.config

        compute_config_dict = {
            key: val for key, val in compute_config.to_dict().items() if val is not None
        }
        formatted_config_json = json.dumps(compute_config_dict, indent=2)
        print(formatted_config_json)



from typing import List, Optional

from anyscale._private.anyscale_client import AnyscaleClientInterface
from anyscale._private.sdk import sdk_docs
from anyscale._private.sdk.base_sdk import Timer
from anyscale.cli_logger import BlockLogger
from anyscale.user_group._private.user_group_sdk import PrivateUserGroupSDK
from anyscale.user_group.commands import (
    _GET_ARG_DOCSTRINGS,
    _GET_EXAMPLE,
    _LIST_ARG_DOCSTRINGS,
    _LIST_EXAMPLE,
    get as get,
    list as list,  # noqa: A004 - claude_comment("claude-opus-4-5", "SDK public API re-export")
)
from anyscale.user_group.models import UserGroup


class UserGroupSDK:
    def __init__(
        self,
        *,
        client: Optional[AnyscaleClientInterface] = None,
        logger: Optional[BlockLogger] = None,
        timer: Optional[Timer] = None,
    ):
        self._private_sdk = PrivateUserGroupSDK(
            client=client, logger=logger, timer=timer
        )

    @sdk_docs(
        doc_py_example=_LIST_EXAMPLE, arg_docstrings=_LIST_ARG_DOCSTRINGS,
    )
    def list(self, *, max_items: int = 50,) -> List[UserGroup]:  # noqa: F811
        """List user groups in the organization.

        Returns a list of UserGroup objects.
        """
        return self._private_sdk.list(max_items=max_items)

    @sdk_docs(
        doc_py_example=_GET_EXAMPLE, arg_docstrings=_GET_ARG_DOCSTRINGS,
    )
    def get(self, id: str) -> UserGroup:  # noqa: F811, A002
        """Get a specific user group by ID.

        Returns a UserGroup object.
        """
        return self._private_sdk.get(group_id=id)



from typing import Any, Dict, List, Optional, Tuple, Union

from anyscale._private.models.model_base import ResultIterator
from anyscale._private.sdk import sdk_command
from anyscale.workspace._private.workspace_sdk import PrivateWorkspaceSDK
from anyscale.workspace.models import (
    UpdateWorkspaceConfig,
    Workspace,
    WorkspaceConfig,
    WorkspaceSortField,
    WorkspaceSortOrder,
    WorkspaceState,
)


_WORKSPACE_SDK_SINGLETON_KEY = "workspace_sdk"

_CREATE_EXAMPLE = """
import anyscale
from anyscale.workspace.models import WorkspaceConfig

anyscale.workspace.create(
    WorkspaceConfig(
        name="my-workspace",
        idle_termination_minutes=120,
    ),
)
"""

_CREATE_ARG_DOCSTRINGS = {"config": "The config for defining the workspace."}
_WAIT_TIMEOUT_SECONDS = 1800.0


@sdk_command(
    _WORKSPACE_SDK_SINGLETON_KEY,
    PrivateWorkspaceSDK,
    doc_py_example=_CREATE_EXAMPLE,
    arg_docstrings=_CREATE_ARG_DOCSTRINGS,
)
def create(
    config: WorkspaceConfig, *, _private_sdk: Optional[PrivateWorkspaceSDK] = None
) -> str:
    """Create a workspace.

    Returns the id of the created workspace.
    """
    return _private_sdk.create(config)  # type: ignore


_START_EXAMPLE = """
import anyscale

anyscale.workspace.start(
    name="my-workspace",
)
"""

_START_ARG_DOCSTRINGS = {
    "name": "Name of the workspace.",
    "id": "Unique ID of the workspace",
    "cloud": "The Anyscale Cloud to run this workload on. If not provided, the organization default will be used (or, if running in a workspace, the cloud of the workspace).",
    "project": "Named project to use for the workspace. If not provided, the default project for the cloud will be used (or, if running in a workspace, the project of the workspace).",
}


@sdk_command(
    _WORKSPACE_SDK_SINGLETON_KEY,
    PrivateWorkspaceSDK,
    doc_py_example=_START_EXAMPLE,
    arg_docstrings=_START_ARG_DOCSTRINGS,
)
def start(
    *,
    name: Optional[str] = None,
    id: Optional[str] = None,  # noqa: A002
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    _private_sdk: Optional[PrivateWorkspaceSDK] = None,
) -> str:
    """Start a workspace.

    Returns the id of the started workspace.
    """
    return _private_sdk.start(name=name, id=id, cloud=cloud, project=project)  # type: ignore


_TERMINATE_EXAMPLE = """
import anyscale

anyscale.workspace.terminate(
    name="my-workspace",
)
"""

_TERMINATE_ARG_DOCSTRINGS = {
    "name": "Name of the workspace.",
    "id": "Unique ID of the workspace",
    "cloud": "The Anyscale Cloud to run this workload on. If not provided, the organization default will be used (or, if running in a workspace, the cloud of the workspace).",
    "project": "Named project to use for the workspace. If not provided, the default project for the cloud will be used (or, if running in a workspace, the project of the workspace).",
}


@sdk_command(
    _WORKSPACE_SDK_SINGLETON_KEY,
    PrivateWorkspaceSDK,
    doc_py_example=_TERMINATE_EXAMPLE,
    arg_docstrings=_TERMINATE_ARG_DOCSTRINGS,
)
def terminate(
    *,
    name: Optional[str] = None,
    id: Optional[str] = None,  # noqa: A002
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    _private_sdk: Optional[PrivateWorkspaceSDK] = None,
) -> str:
    """Terminate a workspace.

    Returns the id of the terminated workspace.
    """
    return _private_sdk.terminate(name=name, id=id, cloud=cloud, project=project)  # type: ignore


_STATUS_EXAMPLE = """
import anyscale

status = anyscale.workspace.status(
    name="my-workspace",
)
"""

_STATUS_ARG_DOCSTRINGS = {
    "name": "Name of the workspace.",
    "id": "Unique ID of the workspace",
    "cloud": "The Anyscale Cloud to run this workload on. If not provided, the organization default will be used (or, if running in a workspace, the cloud of the workspace).",
    "project": "Named project to use for the workspace. If not provided, the default project for the cloud will be used (or, if running in a workspace, the project of the workspace).",
}


@sdk_command(
    _WORKSPACE_SDK_SINGLETON_KEY,
    PrivateWorkspaceSDK,
    doc_py_example=_STATUS_EXAMPLE,
    arg_docstrings=_STATUS_ARG_DOCSTRINGS,
)
def status(
    *,
    name: Optional[str] = None,
    id: Optional[str] = None,  # noqa: A002
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    _private_sdk: Optional[PrivateWorkspaceSDK] = None,
) -> str:
    """Get the status of a workspace.

    Returns the status of the workspace.
    """
    return _private_sdk.status(name=name, id=id, cloud=cloud, project=project)  # type: ignore


_WAIT_EXAMPLE = """
import anyscale

anyscale.workspace.wait(
    name="my-workspace",
)
"""

_WAIT_ARG_DOCSTRINGS = {
    "name": "Name of the workspace.",
    "id": "Unique ID of the workspace",
    "cloud": "The Anyscale Cloud to run this workload on. If not provided, the organization default will be used (or, if running in a workspace, the cloud of the workspace).",
    "project": "Named project to use for the workspace. If not provided, the default project for the cloud will be used (or, if running in a workspace, the project of the workspace).",
    "timeout_s": "The maximum time to wait for the workspace to reach a terminal state.",
    "state": "The desired terminal state to wait for, defaults to RUNNING.",
}


@sdk_command(
    _WORKSPACE_SDK_SINGLETON_KEY,
    PrivateWorkspaceSDK,
    doc_py_example=_WAIT_EXAMPLE,
    arg_docstrings=_WAIT_ARG_DOCSTRINGS,
)
def wait(
    *,
    name: Optional[str] = None,
    id: Optional[str] = None,  # noqa: A002
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    timeout_s: float = _WAIT_TIMEOUT_SECONDS,
    state: str = WorkspaceState.RUNNING,
    _private_sdk: Optional[PrivateWorkspaceSDK] = None,
) -> str:
    """Wait for a workspace to reach a terminal state.

    Returns the status of the workspace.
    """
    return _private_sdk.wait(  # type: ignore
        name=name,
        id=id,
        cloud=cloud,
        project=project,
        timeout_s=timeout_s,
        state=state,
    )


_GENERATE_SSH_CONFIG_FILE_EXAMPLE = """
import anyscale
import subprocess

host_name, config_file = anyscale.workspace.generate_ssh_config_file(
    name="my-workspace",
)

# run an ssh command using the generated config file
subprocess.run(["ssh", "-F", config_path, host_name, "ray --version"])
"""

_GENERATE_SSH_CONFIG_FILE_ARG_DOCSTRINGS = {
    "name": "Name of the workspace.",
    "id": "Unique ID of the workspace",
    "cloud": "The Anyscale Cloud to run this workload on. If not provided, the organization default will be used (or, if running in a workspace, the cloud of the workspace).",
    "project": "Named project to use for the workspace. If not provided, the default project for the cloud will be used (or, if running in a workspace, the project of the workspace).",
    "ssh_config_path": "The directory to write the generated config file to.",
}


@sdk_command(
    _WORKSPACE_SDK_SINGLETON_KEY,
    PrivateWorkspaceSDK,
    doc_py_example=_GENERATE_SSH_CONFIG_FILE_EXAMPLE,
    arg_docstrings=_GENERATE_SSH_CONFIG_FILE_ARG_DOCSTRINGS,
)
def generate_ssh_config_file(
    *,
    name: Optional[str] = None,
    id: Optional[str] = None,  # noqa: A002
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    ssh_config_path: Optional[str] = None,
    _private_sdk: Optional[PrivateWorkspaceSDK] = None,
) -> Tuple[str, str]:
    """Generate an SSH config file for a workspace.

    Returns the hostname and path to the generated config file.
    """
    return _private_sdk.generate_ssh_config_file(  # type: ignore
        name=name, id=id, cloud=cloud, project=project, ssh_config_path=ssh_config_path,
    )


_RUN_COMMAND_EXAMPLE = """
import anyscale

process = anyscale.workspace.run_command(
    name="my-workspace",
    command="ray_version",
    capture_output=True,
    text=True,
)
print(process.stdout)
"""

_RUN_COMMAND_ARG_DOCSTRINGS = {
    "name": "Name of the workspace.",
    "id": "Unique ID of the workspace",
    "cloud": "The Anyscale Cloud to run this workload on. If not provided, the organization default will be used (or, if running in a workspace, the cloud of the workspace).",
    "project": "Named project to use for the workspace. If not provided, the default project for the cloud will be used (or, if running in a workspace, the project of the workspace).",
    "command": "The command to run.",
    "kwargs": "Additional arguments to pass to subprocess.run.",
}


@sdk_command(
    _WORKSPACE_SDK_SINGLETON_KEY,
    PrivateWorkspaceSDK,
    doc_py_example=_RUN_COMMAND_EXAMPLE,
    arg_docstrings=_RUN_COMMAND_ARG_DOCSTRINGS,
)
def run_command(
    *,
    name: Optional[str] = None,
    id: Optional[str] = None,  # noqa: A002
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    command: str,
    _private_sdk: Optional[PrivateWorkspaceSDK] = None,
    **kwargs: Dict[str, Any],
):
    """Run a command in a workspace.

    Returns a subprocess.CompletedProcess object.
    """
    return _private_sdk.run_command(  # type: ignore
        name=name, id=id, cloud=cloud, project=project, command=command, **kwargs,
    )


_PULL_EXAMPLE = """
import anyscale

anyscale.workspace.pull(
    name="my-workspace",
)
"""

_PULL_ARG_DOCSTRINGS = {
    "name": "Name of the workspace.",
    "id": "Unique ID of the workspace",
    "cloud": "The Anyscale Cloud to run this workload on. If not provided, the organization default will be used (or, if running in a workspace, the cloud of the workspace).",
    "project": "Named project to use for the workspace. If not provided, the default project for the cloud will be used (or, if running in a workspace, the project of the workspace).",
    "local_dir": "The local directory to pull the workspace to. If not provided, the current working directory will be used.",
    "pull_git_state": "Whether to pull the git state of the workspace.",
    "rsync_args": "Additional arguments to pass to rsync.",
    "delete": "Whether to delete files in the local directory that are not in the workspace. Excluded files (e.g., .git or custom exclusions) are preserved and not deleted.",
}


@sdk_command(
    _WORKSPACE_SDK_SINGLETON_KEY,
    PrivateWorkspaceSDK,
    doc_py_example=_PULL_EXAMPLE,
    arg_docstrings=_PULL_ARG_DOCSTRINGS,
)
def pull(
    *,
    name: Optional[str] = None,
    id: Optional[str] = None,  # noqa: A002
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    local_dir: Optional[str] = None,
    pull_git_state: bool = False,
    rsync_args: Optional[List[str]] = None,
    delete: bool = False,
    _private_sdk: Optional[PrivateWorkspaceSDK] = None,
) -> None:
    """Pull a workspace to a local directory.

    New files will be created, existing files will be overwritten. With --delete, files
    in the local directory that don't exist in the workspace will be removed. Excluded
    files (like .git) are preserved and not deleted even with --delete.

    Returns the path to the pulled workspace.
    """
    _private_sdk.pull(  # type: ignore
        name=name,
        id=id,
        cloud=cloud,
        project=project,
        local_dir=local_dir,
        pull_git_state=pull_git_state,
        rsync_args=rsync_args,
        delete=delete,
    )


_PUSH_EXAMPLE = """
import anyscale

anyscale.workspace.push(
    name="my-workspace",
    local_dir="~/workspace",
)
"""

_PUSH_ARG_DOCSTRINGS = {
    "name": "Name of the workspace.",
    "id": "Unique ID of the workspace",
    "cloud": "The Anyscale Cloud to run this workload on. If not provided, the organization default will be used (or, if running in a workspace, the cloud of the workspace).",
    "project": "Named project to use for the workspace. If not provided, the default project for the cloud will be used (or, if running in a workspace, the project of the workspace).",
    "local_dir": "The local directory to push to the workspace. If not provided, the current working directory will be used.",
    "push_git_state": "Whether to push the git state of the workspace.",
    "rsync_args": "Additional arguments to pass to rsync.",
    "delete": "Whether to delete files in the workspace that are not in the local directory. Excluded files (e.g., .git or custom exclusions) are preserved and not deleted.",
}


@sdk_command(
    _WORKSPACE_SDK_SINGLETON_KEY,
    PrivateWorkspaceSDK,
    doc_py_example=_PUSH_EXAMPLE,
    arg_docstrings=_PUSH_ARG_DOCSTRINGS,
)
def push(
    *,
    name: Optional[str] = None,
    id: Optional[str] = None,  # noqa: A002
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    local_dir: Optional[str] = None,
    push_git_state: bool = False,
    rsync_args: Optional[List[str]] = None,
    delete: bool = False,
    _private_sdk: Optional[PrivateWorkspaceSDK] = None,
) -> None:
    """Push a local directory to a workspace.

    New files will be created, existing files will be overwritten. With --delete, files
    in the workspace that don't exist locally will be removed. Excluded files (like .git)
    are preserved and not deleted even with --delete.

    Returns the path to the pushed workspace.
    """
    _private_sdk.push(  # type: ignore
        name=name,
        id=id,
        cloud=cloud,
        project=project,
        local_dir=local_dir,
        push_git_state=push_git_state,
        rsync_args=rsync_args,
        delete=delete,
    )


_UPDATE_EXAMPLE = """
import anyscale

anyscale.workspace.update(
    id="<workspace-id>",
    config=UpdateWorkspaceConfig(
        name="new-workspace-name",
        idle_termination_minutes=120,
    ),
)
"""

_UPDATE_ARG_DOCSTRINGS = {
    "id": "Unique ID of the workspace",
    "config": "The config for updating the workspace. Unspecified fields will retain their current values, while specified fields will be updated.",
}


@sdk_command(
    _WORKSPACE_SDK_SINGLETON_KEY,
    PrivateWorkspaceSDK,
    doc_py_example=_UPDATE_EXAMPLE,
    arg_docstrings=_UPDATE_ARG_DOCSTRINGS,
)
def update(
    *,
    id: Optional[str] = None,  # noqa: A002
    config: UpdateWorkspaceConfig,
    _private_sdk: Optional[PrivateWorkspaceSDK] = None,
) -> None:
    """Update a workspace."""
    _private_sdk.update(  # type: ignore
        id=id, config=config,
    )


_GET_EXAMPLE = """
import anyscale
from anyscale.workspace.models import Workspace

workspace: Workspace = anyscale.workspace.get(
    name='my-workspace',
)
"""

_GET_ARG_DOCSTRINGS = {
    "name": "Name of the workspace.",
    "id": "Unique ID of the workspace",
    "cloud": "The Anyscale Cloud to run this workload on. If not provided, the organization default will be used (or, if running in a workspace, the cloud of the workspace).",
    "project": "Named project to use for the workspace. If not provided, the default project for the cloud will be used (or, if running in a workspace, the project of the workspace).",
}


@sdk_command(
    _WORKSPACE_SDK_SINGLETON_KEY,
    PrivateWorkspaceSDK,
    doc_py_example=_GET_EXAMPLE,
    arg_docstrings=_GET_ARG_DOCSTRINGS,
)
def get(
    *,
    name: Optional[str] = None,
    id: Optional[str] = None,  # noqa: A002
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    include_config: bool = True,
    _private_sdk: Optional[PrivateWorkspaceSDK] = None,
) -> Workspace:
    """Get a workspace.

    Args:
        include_config: If True (default), fetch full workspace config. Set to False for efficiency.
    """
    return _private_sdk.get(name=name, id=id, cloud=cloud, project=project, include_config=include_config)  # type: ignore


_LIST_EXAMPLE = """
import anyscale
from anyscale.workspace.models import Workspace

for workspace in anyscale.workspace.list(max_items=10):
    print(f"{workspace.name}: {workspace.state}")
"""

_LIST_ARG_DOCSTRINGS = {
    "name": "Filter by workspace name (substring match).",
    "workspace_id": "Fetch a single workspace by ID.",
    "project": "Named project to filter by.",
    "cloud": "Named cloud to filter by.",
    "creator_id": "Filter workspaces by creator user ID.",
    "state_filter": "List of states to include. May be WorkspaceState enums or case-insensitive strings.",
    "tags_filter": "Filter by tags. Dict mapping tag keys to lists of values. Tags with the same key are ORed, different keys are ANDed.",
    "include_config": "If True, fetch full config for each workspace (expensive). Defaults to False for efficiency.",
    "sort_field": "Field to sort by (STATUS, CREATED_AT, LATEST_STARTED_AT). Defaults to status then created_at.",
    "sort_order": "Sort order (ASC or DESC). Defaults to appropriate order for the sort field.",
    "max_items": "Maximum number of items to return. If None, all items are returned.",
    "page_size": "Number of items to fetch per API call (affects performance, not total results).",
}


@sdk_command(
    _WORKSPACE_SDK_SINGLETON_KEY,
    PrivateWorkspaceSDK,
    doc_py_example=_LIST_EXAMPLE,
    arg_docstrings=_LIST_ARG_DOCSTRINGS,
)
def list(  # noqa: A001, PLR0913, PLR0917
    *,
    workspace_id: Optional[str] = None,
    name: Optional[str] = None,
    project: Optional[str] = None,
    cloud: Optional[str] = None,
    creator_id: Optional[str] = None,
    state_filter: Optional[Union[List[WorkspaceState], List[str]]] = None,
    tags_filter: Optional[Dict[str, List[str]]] = None,
    include_config: bool = False,
    sort_field: Optional[Union[str, WorkspaceSortField]] = None,
    sort_order: Optional[Union[str, WorkspaceSortOrder]] = None,
    max_items: Optional[int] = None,
    page_size: Optional[int] = None,
    _private_sdk: Optional[PrivateWorkspaceSDK] = None,
) -> ResultIterator[Workspace]:
    """List workspaces with optional filters.

    Returns an iterator of Workspace objects. By default, filters to non-terminated states
    and does not fetch the workspace config data.
    """
    return _private_sdk.list(  # type: ignore
        workspace_id=workspace_id,
        name=name,
        project=project,
        cloud=cloud,
        creator_id=creator_id,
        state_filter=state_filter,
        tags_filter=tags_filter,
        include_config=include_config,
        sort_field=sort_field,
        sort_order=sort_order,
        max_items=max_items,
        page_size=page_size,
    )


# Workspace tag operations (SDK)
_TAGS_ADD_EXAMPLE = """
import anyscale

anyscale.workspace.add_tags(id="ws_123", tags={"team": "mlops", "env": "prod"})
"""

_TAGS_ADD_ARG_DOCSTRINGS = {
    "id": "ID of the workspace. Provide either id or name.",
    "name": "Name of the workspace. Provide either id or name.",
    "cloud": "Cloud name (used when resolving by name).",
    "project": "Project name (used when resolving by name).",
    "tags": "Key/value tags to upsert as a map {key: value}.",
}


@sdk_command(
    _WORKSPACE_SDK_SINGLETON_KEY,
    PrivateWorkspaceSDK,
    doc_py_example=_TAGS_ADD_EXAMPLE,
    arg_docstrings=_TAGS_ADD_ARG_DOCSTRINGS,
)
def add_tags(
    *,
    id: Optional[str] = None,  # noqa: A002
    name: Optional[str] = None,
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    tags: Dict[str, str],
    _private_sdk: Optional[PrivateWorkspaceSDK] = None,
):
    """Upsert (add/update) tag key/value pairs for a workspace."""
    return _private_sdk.add_tags(  # type: ignore
        id=id, name=name, cloud=cloud, project=project, tags=tags
    )


_TAGS_REMOVE_EXAMPLE = """
import anyscale

anyscale.workspace.remove_tags(id="ws_123", keys=["team", "env"])
"""

_TAGS_REMOVE_ARG_DOCSTRINGS = {
    "id": "ID of the workspace. Provide either id or name.",
    "name": "Name of the workspace. Provide either id or name.",
    "cloud": "Cloud name (used when resolving by name).",
    "project": "Project name (used when resolving by name).",
    "keys": "List of tag keys to remove.",
}


@sdk_command(
    _WORKSPACE_SDK_SINGLETON_KEY,
    PrivateWorkspaceSDK,
    doc_py_example=_TAGS_REMOVE_EXAMPLE,
    arg_docstrings=_TAGS_REMOVE_ARG_DOCSTRINGS,
)
def remove_tags(
    *,
    id: Optional[str] = None,  # noqa: A002
    name: Optional[str] = None,
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    keys: List[str],
    _private_sdk: Optional[PrivateWorkspaceSDK] = None,
):
    """Remove tags by key from a workspace."""
    return _private_sdk.remove_tags(  # type: ignore
        id=id, name=name, cloud=cloud, project=project, keys=keys
    )


_TAGS_LIST_EXAMPLE = """
import anyscale

tags: dict[str, str] = anyscale.workspace.list_tags(name="my-workspace")
"""

_TAGS_LIST_ARG_DOCSTRINGS = {
    "id": "ID of the workspace. Provide either id or name.",
    "name": "Name of the workspace. Provide either id or name.",
    "cloud": "Cloud name (used when resolving by name).",
    "project": "Project name (used when resolving by name).",
}


@sdk_command(
    _WORKSPACE_SDK_SINGLETON_KEY,
    PrivateWorkspaceSDK,
    doc_py_example=_TAGS_LIST_EXAMPLE,
    arg_docstrings=_TAGS_LIST_ARG_DOCSTRINGS,
)
def list_tags(
    *,
    id: Optional[str] = None,  # noqa: A002
    name: Optional[str] = None,
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    _private_sdk: Optional[PrivateWorkspaceSDK] = None,
) -> Dict[str, str]:
    """List tags for a workspace as a key/value mapping."""
    return _private_sdk.list_tags(  # type: ignore
        id=id, name=name, cloud=cloud, project=project
    )



# coding: utf-8

"""
    Anyscale API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from anyscale_client.configuration import Configuration


class ClusterComputeConfig(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'cloud_id': 'str',
        'maximum_uptime_minutes': 'int',
        'deployment_configs': 'list[CloudDeploymentComputeConfig]',
        'max_workers': 'int',
        'region': 'str',
        'allowed_azs': 'list[str]',
        'head_node_type': 'ComputeNodeType',
        'worker_node_types': 'list[WorkerNodeType]',
        'aws_advanced_configurations_json': 'object',
        'gcp_advanced_configurations_json': 'object',
        'advanced_configurations_json': 'object',
        'auto_select_worker_config': 'bool',
        'flags': 'object',
        'idle_termination_minutes': 'int'
    }

    attribute_map = {
        'cloud_id': 'cloud_id',
        'maximum_uptime_minutes': 'maximum_uptime_minutes',
        'deployment_configs': 'deployment_configs',
        'max_workers': 'max_workers',
        'region': 'region',
        'allowed_azs': 'allowed_azs',
        'head_node_type': 'head_node_type',
        'worker_node_types': 'worker_node_types',
        'aws_advanced_configurations_json': 'aws_advanced_configurations_json',
        'gcp_advanced_configurations_json': 'gcp_advanced_configurations_json',
        'advanced_configurations_json': 'advanced_configurations_json',
        'auto_select_worker_config': 'auto_select_worker_config',
        'flags': 'flags',
        'idle_termination_minutes': 'idle_termination_minutes'
    }

    def __init__(self, cloud_id=None, maximum_uptime_minutes=None, deployment_configs=None, max_workers=None, region=None, allowed_azs=None, head_node_type=None, worker_node_types=None, aws_advanced_configurations_json=None, gcp_advanced_configurations_json=None, advanced_configurations_json=None, auto_select_worker_config=False, flags=None, idle_termination_minutes=None, local_vars_configuration=None):  # noqa: E501
        """ClusterComputeConfig - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._cloud_id = None
        self._maximum_uptime_minutes = None
        self._deployment_configs = None
        self._max_workers = None
        self._region = None
        self._allowed_azs = None
        self._head_node_type = None
        self._worker_node_types = None
        self._aws_advanced_configurations_json = None
        self._gcp_advanced_configurations_json = None
        self._advanced_configurations_json = None
        self._auto_select_worker_config = None
        self._flags = None
        self._idle_termination_minutes = None
        self.discriminator = None

        self.cloud_id = cloud_id
        if maximum_uptime_minutes is not None:
            self.maximum_uptime_minutes = maximum_uptime_minutes
        if deployment_configs is not None:
            self.deployment_configs = deployment_configs
        if max_workers is not None:
            self.max_workers = max_workers
        if region is not None:
            self.region = region
        if allowed_azs is not None:
            self.allowed_azs = allowed_azs
        self.head_node_type = head_node_type
        if worker_node_types is not None:
            self.worker_node_types = worker_node_types
        if aws_advanced_configurations_json is not None:
            self.aws_advanced_configurations_json = aws_advanced_configurations_json
        if gcp_advanced_configurations_json is not None:
            self.gcp_advanced_configurations_json = gcp_advanced_configurations_json
        if advanced_configurations_json is not None:
            self.advanced_configurations_json = advanced_configurations_json
        if auto_select_worker_config is not None:
            self.auto_select_worker_config = auto_select_worker_config
        if flags is not None:
            self.flags = flags
        if idle_termination_minutes is not None:
            self.idle_termination_minutes = idle_termination_minutes

    @property
    def cloud_id(self):
        """Gets the cloud_id of this ClusterComputeConfig.  # noqa: E501

        The ID of the Anyscale cloud to use for launching Clusters.  # noqa: E501

        :return: The cloud_id of this ClusterComputeConfig.  # noqa: E501
        :rtype: str
        """
        return self._cloud_id

    @cloud_id.setter
    def cloud_id(self, cloud_id):
        """Sets the cloud_id of this ClusterComputeConfig.

        The ID of the Anyscale cloud to use for launching Clusters.  # noqa: E501

        :param cloud_id: The cloud_id of this ClusterComputeConfig.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and cloud_id is None:  # noqa: E501
            raise ValueError("Invalid value for `cloud_id`, must not be `None`")  # noqa: E501

        self._cloud_id = cloud_id

    @property
    def maximum_uptime_minutes(self):
        """Gets the maximum_uptime_minutes of this ClusterComputeConfig.  # noqa: E501

        If set to a positive number, Anyscale will terminate the cluster this many minutes after cluster start.  # noqa: E501

        :return: The maximum_uptime_minutes of this ClusterComputeConfig.  # noqa: E501
        :rtype: int
        """
        return self._maximum_uptime_minutes

    @maximum_uptime_minutes.setter
    def maximum_uptime_minutes(self, maximum_uptime_minutes):
        """Sets the maximum_uptime_minutes of this ClusterComputeConfig.

        If set to a positive number, Anyscale will terminate the cluster this many minutes after cluster start.  # noqa: E501

        :param maximum_uptime_minutes: The maximum_uptime_minutes of this ClusterComputeConfig.  # noqa: E501
        :type: int
        """

        self._maximum_uptime_minutes = maximum_uptime_minutes

    @property
    def deployment_configs(self):
        """Gets the deployment_configs of this ClusterComputeConfig.  # noqa: E501

        A list of cloud deployment-specific configs to use.  # noqa: E501

        :return: The deployment_configs of this ClusterComputeConfig.  # noqa: E501
        :rtype: list[CloudDeploymentComputeConfig]
        """
        return self._deployment_configs

    @deployment_configs.setter
    def deployment_configs(self, deployment_configs):
        """Sets the deployment_configs of this ClusterComputeConfig.

        A list of cloud deployment-specific configs to use.  # noqa: E501

        :param deployment_configs: The deployment_configs of this ClusterComputeConfig.  # noqa: E501
        :type: list[CloudDeploymentComputeConfig]
        """

        self._deployment_configs = deployment_configs

    @property
    def max_workers(self):
        """Gets the max_workers of this ClusterComputeConfig.  # noqa: E501

        DEPRECATED. This attribute will be ignored - please use the \"max_resources\" flag to configure resource limits.  # noqa: E501

        :return: The max_workers of this ClusterComputeConfig.  # noqa: E501
        :rtype: int
        """
        return self._max_workers

    @max_workers.setter
    def max_workers(self, max_workers):
        """Sets the max_workers of this ClusterComputeConfig.

        DEPRECATED. This attribute will be ignored - please use the \"max_resources\" flag to configure resource limits.  # noqa: E501

        :param max_workers: The max_workers of this ClusterComputeConfig.  # noqa: E501
        :type: int
        """

        self._max_workers = max_workers

    @property
    def region(self):
        """Gets the region of this ClusterComputeConfig.  # noqa: E501

        The region to launch Clusters in, e.g. \"us-west-2\".  # noqa: E501

        :return: The region of this ClusterComputeConfig.  # noqa: E501
        :rtype: str
        """
        return self._region

    @region.setter
    def region(self, region):
        """Sets the region of this ClusterComputeConfig.

        The region to launch Clusters in, e.g. \"us-west-2\".  # noqa: E501

        :param region: The region of this ClusterComputeConfig.  # noqa: E501
        :type: str
        """

        self._region = region

    @property
    def allowed_azs(self):
        """Gets the allowed_azs of this ClusterComputeConfig.  # noqa: E501

        The availability zones that sessions are allowed to be launched in, e.g. \"us-west-2a\". If not specified or \"any\" is provided as the option, any AZ may be used. If \"any\" is provided, it must be the only item in the list.  # noqa: E501

        :return: The allowed_azs of this ClusterComputeConfig.  # noqa: E501
        :rtype: list[str]
        """
        return self._allowed_azs

    @allowed_azs.setter
    def allowed_azs(self, allowed_azs):
        """Sets the allowed_azs of this ClusterComputeConfig.

        The availability zones that sessions are allowed to be launched in, e.g. \"us-west-2a\". If not specified or \"any\" is provided as the option, any AZ may be used. If \"any\" is provided, it must be the only item in the list.  # noqa: E501

        :param allowed_azs: The allowed_azs of this ClusterComputeConfig.  # noqa: E501
        :type: list[str]
        """

        self._allowed_azs = allowed_azs

    @property
    def head_node_type(self):
        """Gets the head_node_type of this ClusterComputeConfig.  # noqa: E501

        Node configuration to use for the head node.   # noqa: E501

        :return: The head_node_type of this ClusterComputeConfig.  # noqa: E501
        :rtype: ComputeNodeType
        """
        return self._head_node_type

    @head_node_type.setter
    def head_node_type(self, head_node_type):
        """Sets the head_node_type of this ClusterComputeConfig.

        Node configuration to use for the head node.   # noqa: E501

        :param head_node_type: The head_node_type of this ClusterComputeConfig.  # noqa: E501
        :type: ComputeNodeType
        """
        if self.local_vars_configuration.client_side_validation and head_node_type is None:  # noqa: E501
            raise ValueError("Invalid value for `head_node_type`, must not be `None`")  # noqa: E501

        self._head_node_type = head_node_type

    @property
    def worker_node_types(self):
        """Gets the worker_node_types of this ClusterComputeConfig.  # noqa: E501

        A list of node types to use for worker nodes.   # noqa: E501

        :return: The worker_node_types of this ClusterComputeConfig.  # noqa: E501
        :rtype: list[WorkerNodeType]
        """
        return self._worker_node_types

    @worker_node_types.setter
    def worker_node_types(self, worker_node_types):
        """Sets the worker_node_types of this ClusterComputeConfig.

        A list of node types to use for worker nodes.   # noqa: E501

        :param worker_node_types: The worker_node_types of this ClusterComputeConfig.  # noqa: E501
        :type: list[WorkerNodeType]
        """

        self._worker_node_types = worker_node_types

    @property
    def aws_advanced_configurations_json(self):
        """Gets the aws_advanced_configurations_json of this ClusterComputeConfig.  # noqa: E501

        [DEPRECATED: use advanced_configurations_json instead] The advanced configuration json that we pass directly AWS APIs when launching an instance. We may do some validation on this json and reject the json if it is using a configuration that Anyscale does not support.  # noqa: E501

        :return: The aws_advanced_configurations_json of this ClusterComputeConfig.  # noqa: E501
        :rtype: object
        """
        return self._aws_advanced_configurations_json

    @aws_advanced_configurations_json.setter
    def aws_advanced_configurations_json(self, aws_advanced_configurations_json):
        """Sets the aws_advanced_configurations_json of this ClusterComputeConfig.

        [DEPRECATED: use advanced_configurations_json instead] The advanced configuration json that we pass directly AWS APIs when launching an instance. We may do some validation on this json and reject the json if it is using a configuration that Anyscale does not support.  # noqa: E501

        :param aws_advanced_configurations_json: The aws_advanced_configurations_json of this ClusterComputeConfig.  # noqa: E501
        :type: object
        """

        self._aws_advanced_configurations_json = aws_advanced_configurations_json

    @property
    def gcp_advanced_configurations_json(self):
        """Gets the gcp_advanced_configurations_json of this ClusterComputeConfig.  # noqa: E501

        [DEPRECATED: use advanced_configurations_json instead] The advanced configuration json that we pass directly GCP APIs when launching an instance. We may do some validation on this json and reject the json if it is using a configuration that Anyscale does not support.  # noqa: E501

        :return: The gcp_advanced_configurations_json of this ClusterComputeConfig.  # noqa: E501
        :rtype: object
        """
        return self._gcp_advanced_configurations_json

    @gcp_advanced_configurations_json.setter
    def gcp_advanced_configurations_json(self, gcp_advanced_configurations_json):
        """Sets the gcp_advanced_configurations_json of this ClusterComputeConfig.

        [DEPRECATED: use advanced_configurations_json instead] The advanced configuration json that we pass directly GCP APIs when launching an instance. We may do some validation on this json and reject the json if it is using a configuration that Anyscale does not support.  # noqa: E501

        :param gcp_advanced_configurations_json: The gcp_advanced_configurations_json of this ClusterComputeConfig.  # noqa: E501
        :type: object
        """

        self._gcp_advanced_configurations_json = gcp_advanced_configurations_json

    @property
    def advanced_configurations_json(self):
        """Gets the advanced_configurations_json of this ClusterComputeConfig.  # noqa: E501

        Advanced configurations for this compute node type to pass to the cloud provider when launching this instance.  # noqa: E501

        :return: The advanced_configurations_json of this ClusterComputeConfig.  # noqa: E501
        :rtype: object
        """
        return self._advanced_configurations_json

    @advanced_configurations_json.setter
    def advanced_configurations_json(self, advanced_configurations_json):
        """Sets the advanced_configurations_json of this ClusterComputeConfig.

        Advanced configurations for this compute node type to pass to the cloud provider when launching this instance.  # noqa: E501

        :param advanced_configurations_json: The advanced_configurations_json of this ClusterComputeConfig.  # noqa: E501
        :type: object
        """

        self._advanced_configurations_json = advanced_configurations_json

    @property
    def auto_select_worker_config(self):
        """Gets the auto_select_worker_config of this ClusterComputeConfig.  # noqa: E501

        If set to true, worker node groups will automatically be selected based on workload.  # noqa: E501

        :return: The auto_select_worker_config of this ClusterComputeConfig.  # noqa: E501
        :rtype: bool
        """
        return self._auto_select_worker_config

    @auto_select_worker_config.setter
    def auto_select_worker_config(self, auto_select_worker_config):
        """Sets the auto_select_worker_config of this ClusterComputeConfig.

        If set to true, worker node groups will automatically be selected based on workload.  # noqa: E501

        :param auto_select_worker_config: The auto_select_worker_config of this ClusterComputeConfig.  # noqa: E501
        :type: bool
        """

        self._auto_select_worker_config = auto_select_worker_config

    @property
    def flags(self):
        """Gets the flags of this ClusterComputeConfig.  # noqa: E501

        A set of advanced cluster-level flags that can be used to configure a particular workload.  # noqa: E501

        :return: The flags of this ClusterComputeConfig.  # noqa: E501
        :rtype: object
        """
        return self._flags

    @flags.setter
    def flags(self, flags):
        """Sets the flags of this ClusterComputeConfig.

        A set of advanced cluster-level flags that can be used to configure a particular workload.  # noqa: E501

        :param flags: The flags of this ClusterComputeConfig.  # noqa: E501
        :type: object
        """

        self._flags = flags

    @property
    def idle_termination_minutes(self):
        """Gets the idle_termination_minutes of this ClusterComputeConfig.  # noqa: E501

        If set to a positive number, Anyscale will terminate the cluster this many minutes after the cluster is idle. Idle time is defined as the time during which a Cluster is not running a user command or a Ray driver. Time spent running commands on Jupyter or ssh is still considered 'idle'. To disable, set this field to 0.  # noqa: E501

        :return: The idle_termination_minutes of this ClusterComputeConfig.  # noqa: E501
        :rtype: int
        """
        return self._idle_termination_minutes

    @idle_termination_minutes.setter
    def idle_termination_minutes(self, idle_termination_minutes):
        """Sets the idle_termination_minutes of this ClusterComputeConfig.

        If set to a positive number, Anyscale will terminate the cluster this many minutes after the cluster is idle. Idle time is defined as the time during which a Cluster is not running a user command or a Ray driver. Time spent running commands on Jupyter or ssh is still considered 'idle'. To disable, set this field to 0.  # noqa: E501

        :param idle_termination_minutes: The idle_termination_minutes of this ClusterComputeConfig.  # noqa: E501
        :type: int
        """

        self._idle_termination_minutes = idle_termination_minutes

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ClusterComputeConfig):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ClusterComputeConfig):
            return True

        return self.to_dict() != other.to_dict()
